C
C      Copyright (c) 1986, 1988 Masatake Mori, All rights Reserved
C      This routine is appeared in
C      "FORTRAN 77 Suuchi Keisan Programing (in Japanese)"
C      Pubulished by Iwanami Shoten, 1986
C
      SUBROUTINE DEFINT(FUNC,A,B,EPS,L,V)
C
C     INTEGRATE FUNC(L=0) OR F(L=1) OVER FINITE INTERVAL (A,B) BY
C     DE FORMULA IN DOUBLE PRECISION
C
C     BEFORE THE FIRST CALL OF THIS ROUTINE FIAB MUST BE CALLED
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C   FUNC: NAME OF FUNCTION SUBPROGRAM FOR INTEGRAND
C   A   : LOWER BOUND OF INTEGRATION
C   B   : UPPER BOUND OF INTEGRATION
C   EPS : ABSOLUTE ERROR TOLERANCE
C   L   = 0 THEN INTEGRATE FUNCTION OVER (A,B)
C       = 1 THEN INTEGRATE F(X) OVER (A,B)
C
C     IN CASE OF L = 1 FUNC MUST BE DEFINED AS FOLLOWS:
C
C     IF (-C .LT. Y .LT. 0) THEN
C       FUNC(Y) = FUNC(B-Y)
C     IF (0 .LT. Y .LE. C) THEN
C       FUNC(Y) = F(A-Y)
C
C       WHERE C = (B - A) / 2
C
C   V   : INTEGRAL OF FUNC (L = 0) OR F(L=1)
C
      DOUBLE PRECISION FUNC,A,B,EPS,V
      DOUBLE PRECISION HALF
      DOUBLE PRECISION H,FAC,SHFM,SHFP
      DOUBLE PRECISION VOLD,VNEW,WM,WP
      DOUBLE PRECISION EPS0,EPSQ,EPSV,EPSM,EPSP
      DOUBLE PRECISION AM,A0,AP,B0,BB
C
      COMMON /CMDEFW/AM(608,2), A0(2), AP(608,2), B0, BB(608)
      COMMON /CMDEFN/NPOW, NEND
C
      DATA HALF /0.5D0 /
      DATA EPS0 /1.0D-32/
      DATA EPSM, EPSP / 0, 0 / 
C
      IF (L .NE. 0 .AND. L .NE. 1) THEN
        V = 0.0
        RETURN
      END IF
C
      FAC = (B-A) * HALF
      IF (L .EQ. 0) THEN
        SHFM = (B+A) * HALF
        SHFP = SHFM
      ELSE
        SHFM = 0.0
        SHFP = 0.0
      END IF
C
      L1 = L+1
C
      IF (ABS(EPS) .GE. EPS0) THEN
        EPSV = ABS(EPS)
      ELSE
        EPSV = EPS0
      END IF
C
      EPSQ = 0.2 * SQRT(EPSV)
C
      H = HALF
C
      IS = 2**NPOW
      IM = IS
C
      KM = 0
      KP = 0
      NM = 0
      NP = 0
C
      VNEW = FUNC(A0(L1)*FAC+SHFP) * B0 
C
C     ---- INITIAL STEP ----
C
C        INTEGRATE WITH MESH SIZE = 0.5
C        AND CHECK DECAY OF INTEGRAND
C
      DO 10 I = IS, NEND, IM
C
        IF (KM .LE. 1) THEN
          WM = FUNC(AM(I,L1) * FAC + SHFM) * BB(I)
          VNEW = VNEW + WM
          IF (ABS(WM) .LE. EPSV) THEN
            KM = KM + 1
            IF (KM .GE. 2) NM = I - IM 
          ELSE
            KM = 0
          END IF
        END IF
C
        IF (KP .LE. 1) THEN
          WP = FUNC(AP(I,L1) * FAC + SHFP) * BB(I)
          VNEW = VNEW + WP
          IF (ABS(WP) .LE. EPSV) THEN
            KP = KP + 1
            IF (KP .GE. 2) NP = I - IM
          ELSE
            KP = 0
          END IF
        END IF
C
        IF (KM .EQ. 2 .AND. KP .EQ. 2) GO TO 11
C
   10 CONTINUE 
C
   11 CONTINUE
C
      IF (NM .EQ. 0) THEN
        NM = NEND
        EPSM = SQRT(ABS(WM))
        WRITE (*,2001)
 2001   FORMAT (' (DEFINT) slow decay on negative side ')
      END IF
C
      IF (NP .EQ. 0) THEN
        NM = NEND
        EPSP = SQRT(ABS(WP))
        WRITE (*,2002)
 2002   FORMAT (' (DEFINT) slow decay on positive side')
      END IF
C
      EPSQ = MAX(EPSQ, EPSM, EPSP)
C
      VOLD = H * FAC * VNEW
C
      DO 20 MSTEP = 1, NPOW
C
        VNEW = 0.0
        IH = IS
        IS = IS / 2
C
        DO 540 I = IS, NM, IH
          VNEW = VNEW + FUNC(AM(I,L1) * FAC + SHFM) * BB(I)
  540   CONTINUE
*
        DO 550 I = IS, NP, IH
          VNEW = VNEW + FUNC(AP(I,L1) * FAC + SHFP) * BB(I)
  550   CONTINUE
C
        VNEW = (VOLD + H * FAC * VNEW) * HALF
C
        IF (ABS(VNEW - VOLD) .LT. EPSQ) THEN
          V = VNEW
          RETURN
        END IF
C
        H = H * HALF
        VOLD = VNEW
C
   20 CONTINUE
C
      WRITE (*,2003) 
 2003 FORMAT (' (DEFINT) insufficient mesh refinement')
      V = VNEW
      RETURN
      END
C
C
C
      SUBROUTINE FIAB
C
C     GENERATE POINTS AND WEIGHTS OF DEF FORMULA
C     FOR DEFINT
C     IN DOUBLE PRECISION
C
      DOUBLE PRECISION ONE,HALF
      DOUBLE PRECISION A9,PH
      DOUBLE PRECISION H,EH,EN,ENI
      DOUBLE PRECISION SH,CH,CHSI,EXS,EXSI
      DOUBLE PRECISION AM,A0,AP,B0,BB
C
      COMMON /CMDEFW/AM(608,2), A0(2), AP(608,2), B0, BB(608)
      COMMON /CMDEFN/NPOW, NEND
C
      PARAMETER (ONE=1.)
      PARAMETER (HALF = ONE /2)
      PARAMETER (N6 = 6)
      PARAMETER (H = ONE / 2**(N6+1))
      A9 = 0.99999 99999 99999 8 D0
C
C     START COMPUTATION OF POINTS AND WEIGHTS
C
      PH = 2 * ATAN(ONE)
      NPOW = N6
      NEND = 608
      A0(1) = 0.0
      A0(2) = 1.0
      B0 = PH
      EH = EXP(H)
      EN = 1.0
C
      DO 10 I = 1, NEND
        EN = EH * EN
        ENI = 1.0 / EN
        SH = (EN - ENI) * HALF
        CH = (EN + ENI) * HALF
        EXS = EXP(PH * SH)
        EXSI = 1.0 / EXS
        CHSI = 2.0 / (EXS + EXSI)
        AP(I,1) = ((EXS - EXSI) * HALF) * CHSI
        IF (AP(I,1) .GE. A9) AP(I,1) = A9
        AP(I,2) = EXSI * CHSI
        AM(I,1) = - AP(I,1)
        AM(I,2) = - AP(I,2)
        BB(I) = PH * CH * CHSI**2
   10 CONTINUE
C
      AP(608,2) = AP(607,2)
      AM(608,2) = AM(607,2)
C
      RETURN
      END
