C
C     Copyright (c) 1986, 1988 Masatake Mori, All rights Reserved
C     This routine is appeared in
C     "FORTRAN 77 Suuchi Keisan Programing (in Japanese)"
C     Pubulished by Iwanami Shoten, 1986
C
      SUBROUTINE NC8 (FUNC, A, B, EPS, V, ESTERR)
C
C     Integrate FUNC(X) from A to B by automatic adaptive routine
C     based on the 8-th order Nerton-Cotes formula in double precision
C
C     ==== input ====
C
C     FUNC  .. name of the integrand function subprogram
C     A     .. lower limit of integration
C     B     .. upper limit of integration
C     EPS   .. absolute error tolerance
C
C     ==== output ====
C
C     V     .. an approximate value to the integral
C     ESTERR.. an estimate of absolute error
C
C     ==== output in named common block ====
C
C     NEVAL .. number of function evaluation
C     IXERR .. number of possible trouble points (up to 10)
C     XERR(I). location of the I-th possible trouble point
C
C     ==== internal parameters
C
C     L     .. level number
C     LS    .. interval number
C
      REAL*8 FUNC
      REAL*8 A,B,EPS,V,ESTERR
      REAL*8 X(0:16),F(0:16),W(0:4)
      REAL*8 WN,W0,W1,W2,W3,W4
      REAL*8 XE
C
      REAL*8 HAB,HV
      REAL*8 QL,QR(31)
      REAL*8 XS(8,30),FS(8,30)
      REAL*8 QOLD,QNEW,DV,DIFF,DERR
      REAL*8 XERR
      REAL*8 ONE,HALF
      REAL*8 ETOL,EPSBA
C
      COMMON / COMXER / NEVAL, IXERR, XERR(10)
C
      PARAMETER (ONE = 1, HALF = ONE / 2)
      PARAMETER (WN = 14175)
C
      PARAMETER (W0 =   3956 / WN)
      PARAMETER (W1 =  23552 / WN)
      PARAMETER (W2 =  -3712 / WN)
      PARAMETER (W3 =  41984 / WN)
      PARAMETER (W4 = -18160 / WN)
C
      DATA W / W0, W1, W2, W3, W4 /
      DATA NMAX / 5000 / 
      DATA IEOUT / 10 /
C
      IF (ABS(B-A) .LT. 1.0D-37) THEN
        V = 0.0D0
        RETURN
      END IF
C
      IF (EPS .GT. 1.0D-4)  EPS = 1.0D-4
      IF (EPS .LT. 1.0D-15) EPS = 1.0D-15
C
C     ---- initialization ----
C
      IF (EPS .GT. 1.0D-6) THEN
        LMAX = 18
      ELSE
        LMAX = 30
      END IF
C
      LMAXV = LMAX
      LMAXH = LMAX / 3 - 1
      IXERR = 0
      IER = 0
      XE = 1.0D75
      EPSBA = 0.01D0 * ABS(B-A)
C
      V = 0.0D0
      ESTERR = 0.0D0
C
      HAB = (B-A)/16
      ETOL = EPS / HAB
C
      L = 0
      LS = 1
C
      DV = 0.0D0
C
      DO 510 J = 0, 16, 2
        X(J) = A + HAB * J
  510 CONTINUE
C
      DO 810 J = 0, 16, 2
        F(J) = FUNC(X(J))
  810 CONTINUE
C
      NEVAL = 9
C
      QOLD = W(4) * F(8)
      DO 520 J = 0, 3
        QOLD = QOLD + W(J) * (F(2*J) + F(16 - 2*J))
  520 CONTINUE
      QOLD = 2 * HAB * QOLD
C
C     ---- general step ----
C
  100 CONTINUE
C
      HV = (X(16) - X(0)) / 16
C
      DO 530 J = 1, 15, 2
        X(J) = X(0) + HV * J
  530 CONTINUE
      DO 830 J = 1, 15, 2
        F(J) = FUNC(X(J))
  830 CONTINUE
C
      NEVAL = NEVAL + 8
C
      QL = W(4) * F(4)
      DO 540 J = 0, 3
        QL = QL + W(J) * (F(J) + F(8-J))
  540 CONTINUE
      QL = HV * QL
C
      QR(L+1) = W(4) * F(12)
      DO 550 J = 0, 3
        QR(L+1) = QR(L+1) + W(J) * (F(8+J) + F(16-J))
  550 CONTINUE
      QR(L+1) = HV * QR(L+1)
C
      QNEW = QL + QR(L+1)
C
      DIFF = QNEW - QOLD
      DERR = DIFF / 1023
C
      IF (NEVAL .GT. NMAX) GO TO 902
      IF (IER .GE. 3) GO TO 101
      IF (L .GE. LMAXV) GO TO 901
C
C     ---- local convergence test ----
C
      IF (LS .EQ. 1) THEN
        IF (ABS(DERR) .LE. HV*ETOL) THEN
          V = QNEW + DERR
          ESTERR = ABS(DERR)
          RETURN
        END IF
      ELSE
        IF (ABS(DERR) .LE. HV*ETOL) THEN
          LMAXV = LMAX
          IER = 0
          GO TO 101
        END IF
      END IF
C
C     ---- not converged ----
C          go down to next level (left interval)
C
      L = L + 1
      LS = 2 * LS
C
C     ---- store elements in right interval
C
      DO 560 J = 1, 8
        XS(J,L) = X(J+8)
        FS(J,L) = F(J+8)
  560 CONTINUE
C
      QOLD = QL
      DO 570 J = 8, 1, -1
        X(2*J) = X(J)
        F(2*J) = F(J)
  570 CONTINUE
C
      GO TO 100
C
  101 CONTINUE
C
C     ---- converged ----
C
      V = V + QNEW
      DV = DV + DERR
      ESTERR = ESTERR + ABS(DERR)
C
  102 CONTINUE
      IF (MOD(LS,2) .EQ. 0) THEN
C
C     ---- it was left interval ----
C          go to right interval
C
        LS = LS + 1
C
        QOLD = QR(L)
        X(0) = X(16)
        F(0) = F(16)
        DO 580 J = 1, 8
          X(2*J) = XS(J,L)
          F(2*J) = FS(J,L)
  580   CONTINUE
C
      ELSE
C
C     ---- it was right interval ----
C          go up to upper level
C
        L = L - 1
        LS = (LS - 1) / 2
C
        IF (L .EQ. LMAXH .AND. IER .GE. 3) THEN
          IER = 0
          LMAXV = LMAXH + 1
        END IF
C
        IF (L .EQ. 0) THEN
C
C     ---- computation completed ----
C
          V = V + DV
          RETURN
        END IF
C
        GO TO 102
C
      END IF
C
      GO TO 100
C
C     ---- number of levels attain the maximum ----
C
  901 CONTINUE
      IF (ABS(XE - X(0)) .LT. EPSBA) THEN
        IER = IER + 1
        GO TO 101
      END IF
      WRITE(0,2001) L, X(0)
 2001 FORMAT(' (NC8) level of subdivision attain the maximum ( = ',
     # I2, ' )' /
     # '  at x = ', 1PD13.6, ' ---- computation continued ----')
      XE = X(0)
      IXERR = IXERR + 1
      IF (IXERR .LE. IEOUT) XERR(IXERR) = X(0)
      GO TO 101
C
C     ---- number of function evaluation ----
C          exceed the muximum
C
  902 CONTINUE
      WRITE(0,2002) NEVAL, L, X(0)
 2002 FORMAT(' (NC8) number of function evaluation ( = ', I5, ' )'/
     # '  exceed the maximum on ', I2, '-th level and at x = ',1PD13.6/
     # 6X, '---- computation terminated ----')
      RETURN
C
      END
