      program sbem
c
c     test program for bem()
c
c     VARIABLES:
c       n      ... number of nodes
c       r,z    ... array of (r,z)-coordinates of node end (in m)
c       ns     ... number of sources emitted from the source
c       rs, zs ... (r,z) coordinates of source location
c       snode  ... temporally var. for the area of nodes.
c
c     WORK ARRAYS for LAPACK
c       ibc, bv, bdv, b, bferr, bberr, ipd, iw, dw, ho, hp
c
      integer nmax, n
      parameter(nmax = 100)

      real*8 r(nmax+1), z(nmax+1)
      integer ibc(nmax)
      real*8 bv(nmax), bdv(nmax), b(nmax)
      real*8 bferr(nmax), bberr(nmax)
      real*8 g(nmax,nmax), h(nmax,nmax)
      integer ipd(nmax), iw(nmax)
      real*8 dw(3,nmax)
      real*8 ho(nmax,nmax), hp(nmax,nmax)
      real*8 snode

      integer nsmax, ns
      parameter(nsmax = 100000)
      real*8 rs(nsmax), zs(nsmax)

      integer i

      call readin(n, nmax, r, z, ibc, bv, ns, nsmax, rs, zs)

      call setbemsrc(n, r, z)

      do 5 i = 1, ns
         call bemsrc(rs(i), zs(i))
 5    continue

      call readbemsrc(n, b)
      write(0,*) 'SOURCE SUMMARY'
      do 6 i = 1, n
          write(0,1005) i, b(i)
 6    continue
 1005 format(1x, 'b(', i3, ') = ', f20.6)
        
      call bem (nmax, n, r, z, ibc, bv, bdv,
     $     b, g, h, ipd, bferr, bberr, dw, iw, ho, hp)

      write(6,1000)
 1000 format('# RESULT',/,'# SOURCE')

      write(6,1010) ns, rs(i), zs(i)
 1010 format(I8, 2G20.8) 

      write(6,1020)
 1020 format('# FLUXES', / ,'# node   flux    node area')

      do 30 i = 1, n
         call axarea(r(i), z(i), r(i+1), z(i+1), snode)
         write(6,1030) i, bdv(i), snode
 30   continue
 1030 format(I3, 2G20.8)

      stop
      end
c================================

      subroutine readin(n, nmax, r, z, ibc, bv, ns, nsmax, rs, zs)
c
c     readin data
c
      integer n, nmax
      real*8 r(n+1), z(n+1)
      integer ibc(n)
      real*8 bv(n)
      integer ns, nsmax
      real*8 rs(ns), zs(ns)

      integer i

      read(5, 2000) n, ns
 2000 format(2I10)
      if ( (n .gt. nmax) .or. (ns .gt. nsmax) ) then
         write(0,*) 'parameter too large'
         stop
      endif

      do 10 i = 1, n+1
         read (5,2010) r(i), z(i)
         r(i) = 0.01d0 * r(i)
         z(i) = 0.01d0 * z(i)
 10   continue

      do 20 i = 1, n
         read (5,2020) ibc(i), bv(i)
 20   continue

      read (5, 2010) rs(1), zs(1)
      rs(1) = 0.01d0 * rs(1)
      zs(1) = 0.01d0 * zs(1)
      do 30 i = 2, ns
         rs(i) = rs(1)
         zs(i) = zs(1)
 30   continue

 2010 format(2F13.6)
 2020 format(I3, F13.6)

      do 40 i =1, n
         write(0,1000) i, r(i), z(i), r(i+1), z(i+1),
     $                 ibc(i), bv(i)
 40   continue
 1000 format(1x, i3, '(', F8.4, ',', F8.4, ')-(',
     $                    F8.4, ',', F8.4, ') D0/N1:',
     $                    I2, ' Bv:', F8.4)

      write(0,1010) rs(i), zs(i)
 1010 format(1x, 'source', ': (', f8.2, ',', f8.2, ')', i6)

      return
      end

