c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998-2000 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: bmain.F,v 1.1 2001/03/26 22:40:43 nakano Exp $
c
#include "hisput.h"

      program bemcmp
c
c     main routine of HISPUT
c
c     == params ==
c     BASE
c       seed   ... seed for random generator sgenrand
c       ntrial ... trial number
c
c     PHYSICAL PARAMETERS
c       tgas   ... tempereture of the gas     (K)
c       pgasig ... pressure of the gas (gauge value) (Pa)
c       tgasig ... gas temperature     (gauge assumption) (K)
c       mgas   ... mass of gas atom           (kg)
c       mpar   ...         sputtered particle (kg)
c       rmax   ... maximum paremeter of the col-param (m)
c
c     SCATTERING
c       POTENTIAL PARAMETERS (see sp.f for details)
c         bmpca  ... Born-Mayer Potential parameter, A (eV)
c         bmpcb  ... Born-Mayer Potential parameter, B (angstrom^-1)
c
c       SPLINE FUNCTION PARAMETERS
c         nesca   ... number of nodes along energy axis
c         nbsca   ...                       col-param axis
c         emaxsca ... maximum value of energy axis (eV)
c         eminsca ... minimum
c         bmaxsca ... maximum value of col-param axis (angstrom)
c
c     COLLISION
c       SPLINE FUNCTION PARAMETERS
c         necol   ... number of nodes along energy axis
c         nwcol   ...                       w(prob.)
c         emaxcol ... maximum of energy axis (eV)
c         emincol ... minimum
c         wmincol ... minimum of w axis, below which vg is prop. to w
c         wmaxcol ... maximun of w axis, above which vg is const.
c
c     BOUNDARY
c       nnode  ... number of boundary elements
c       rnode  ... radial positions of nodes of elements
c       znode  ... vertical positions
c
c     SOURCE
c       rej, zej . (r,z) componetns of particle source

      integer seed
      integer itrial, ntrial

      real*8 tgas, ngas
      real*8 mgas, mpar, rmax

      real*8 bmpca, bmpcb
      integer nesca, nbsca
      real*8 emaxsca, eminsca, bmaxsca

      integer necol, nwcol
      real*8 emaxcol, emincol, wmincol, wmaxcol

      integer nnodemax, nnode
      parameter(nnodemax = NODEMAX)
      real*8 rnode(nnodemax+1), znode(nnodemax+1)
      integer ntrap(nnodemax)
      real*8 ftrap(nnodemax), s

      real*8 rej, zej

      real*8 genrand

      real*8 ep, vp(3), xp0(3), xp1(3)
      real*8 vg, vr, fp, w, w2

      integer nidx

      real*8 pi
      parameter (pi = PIDBL)

      integer i

c
c     start:
c
      call setfpu

      call readin(seed, ntrial,
     $     bmpca, bmpcb,
     $     nesca, nbsca, emaxsca, eminsca, bmaxsca,
     $     tgas, ngas, mgas, mpar, rmax,
     $     necol, nwcol, emaxcol, emincol, wmaxcol, wmincol,
     $     nnodemax, nnode, rnode, znode,
     $     rej, zej)

      call sgenrand(seed)
      call inispf1
      call inispf2
      call hiab

      call setsp(bmpca, bmpcb)
      call setsca(nesca, nbsca, emaxsca, eminsca, bmaxsca)
      call setcol(tgas, ngas, mgas, mpar, rmax,
     $     necol, nwcol, emaxcol, emincol, wmaxcol, wmincol)

      call setcdir(mpar, mgas, rmax)
      call settrap(nnode, rnode, znode)

      do 5 nidx = 1, nnode
         ntrap(nidx) = 0
 5    continue

      do 10 itrial = 1, ntrial

         w = 2 * pi * genrand()
         xp0(1) = rej * cos(w)        
         xp0(2) = rej * sin(w)        
         xp0(3) = zej

         w = genrand()
         w2 = 2 * pi * genrand()
         vp(3) = 1.0d0 - 2 * w
         vp(1) = 2 * sqrt(w * (1-w)) * cos(w2)
         vp(2) = 2 * sqrt(w * (1-w)) * sin(w2)
         ep = eminsca

 25      continue
c
c     -- particle path --
c
         w = genrand()
         call frpath(ep, w, fp)

         do 30 i = 1, 3
            xp1(i) = xp0(i) + fp * vp(i)
 30      continue

         call trap(xp0, xp1, nidx)

         if (nidx .ne. 0) then
            ntrap(nidx) = ntrap(nidx) + 1
            go to 10
         end if

         do 32 i = 1, 3
            xp0(i) = xp1(i)
 32      continue

c
c     -- collision --
c
         w = genrand()
         call colgas(ep, w, vg)
         w = genrand()
         call relv(ep, vg, w, vr)
         call cdir(ep, vp, vg, vr)

         go to 25
 10   continue

c
c     -- TRAP summary --
c
      do 100 i = 1, nnode
         call axarea(rnode(i), znode(i), rnode(i+1), znode(i+1), s)
         ftrap(i) = ntrap(i) / (s * ntrial)
 100  continue

      write(6, 1020)
 1020 format('#RESULT',/,'#node   flux')
      do 300 i = 1, nnode
         write(6, 1030) i, ftrap(i)
 300  continue
 1030 format(1x, i4, f20.6)

      stop
      end

