c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: breadin.F,v 1.3 2001/07/24 09:23:42 nakano Exp $
c
#include "hisput.h"

      subroutine readin(seed, ntrial,
     $     bmpca, bmpcb,
     $     nesca, nbsca, emaxsca, eminsca, bmaxsca,
     $     tgas, ngas, mgas, mpar, rmax,
     $     necol, nwcol, emaxcol, emincol, wmaxcol, wmincol,
     $     nnodemax, nnode, rnode, znode,
     $     eb, e1,
     $     npejmax, npej, rpej, dpej, zpej,
     $     ntimemax, ntime, tsnap)

c
c     subroutine for data reading
c
      integer seed, ntrial

      real*8  bmpca, bmpcb 

      integer nesca, nbsca
      real*8 emaxsca, eminsca, bmaxsca
      real*8 tgas, ngas
      real*8 mgas, mpar, rmax
      real*8 unitam, unitbz
      parameter (unitam = UNITAM)
      parameter (unitbz = UNITBZ)
 
      integer necol, nwcol
      real*8 emaxcol, emincol, wmaxcol, wmincol

      integer nnodemax, nnode
      real*8 rnode(nnodemax+1), znode(nnodemax+1)

      real*8 eb, e1

      integer npejmax, npej
      real*8 rpej(npejmax), dpej(npejmax), zpej

      integer ntimemax, ntime
      real*8 tsnap(ntimemax)

      character*20 elmpar, elmgas
      real*8 bmaspt, bmbspt, bmagas, bmbgas
      real*8 pgasig, tgasig, pgas
      integer inode, ipej, itime

 2000 format( i10)
 2010 format( a20)
 2020 format( f14.6)
 2030 format(2f14.6)
 2040 format(3f14.6)

      read(5,2000) seed
      read(5,2000) ntrial

      read(5,2010) elmpar
      read(5,2020) mpar
      read(5,2020) bmaspt
      read(5,2020) bmbspt
      read(5,2010) elmgas
      read(5,2020) mgas
      read(5,2020) bmagas
      read(5,2020) bmbgas
c
c     convert atomic unit -> kg
c
      mpar = mpar * unitam
      mgas = mgas * unitam

      read(5,2020) eb
      read(5,2020) e1

      read(5,2020) tgas
      read(5,2020) pgasig
      read(5,2020) tgasig

      pgas = pgasig * tgas / tgasig
      ngas = pgasig / (unitbz * tgasig)

      read(5,2020) rmax

      read(5,2000) nesca
      read(5,2000) nbsca
      read(5,2020) emaxsca
      read(5,2020) eminsca
      read(5,2020) bmaxsca

      read(5,2000) necol
      read(5,2000) nwcol
      read(5,2020) emaxcol
      read(5,2020) emincol
      read(5,2020) wmaxcol
      read(5,2020) wmincol

      read(5,2000) nnode
      if (nnode .gt. nnodemax) then
         write(6,1900)
         stop
      end if
 1900 format(' (readin) -- boundary node number too large')

      do 10 inode = 1, nnode+1
         read(5,2030) rnode(inode), znode(inode)
         rnode(inode) = rnode(inode) * 1.0d-2
         znode(inode) = znode(inode) * 1.0d-2
 10   continue


      read(5,2000) npej
      if (npej .gt. npejmax) then
         write(6, 1910)
         stop
      end if
 1910 format(' (readin) -- erosion track node number too large')

      do 30 ipej = 1, npej
         read(5,2030) rpej(ipej), dpej(ipej)
         rpej(ipej) = rpej(ipej) * 1.0d-2
         dpej(ipej) = dpej(ipej) * 1.0d-2
 30   continue

c WARN!!! HARDCORD
      zpej = 0.0d0


      read(5,2000) ntime
      if (ntime .gt. ntimemax) then
         write(0, 1930)
         stop
      end if
 1930 format('(readin) -- snap time is too many')

      do 40 itime = 1, ntime
         read(5,2020) tsnap(itime)
 40   continue

c
c     interatomic potential parameters
c
      bmpca = dsqrt(bmagas * bmaspt)
      bmpcb = 0.5d0 * (bmbgas + bmbspt)

c
c     write conditions
c      
 1010 format(1x, 'random seed                               :', i12)
 1020 format(1x, 'trial number                              :', i12)

 1100 format(1x, 'sputterd particle element                 :', a20) 
 1110 format(1x, '   mass                             (kg)  :', e12.6e2)
 1120 format(1x, 'sputtering gas element                    :', a20)
 1130 format(1x, '   mass                             (kg)  :', e12.6e2)
 1140 format(1x, 'BM potential constant a             (ev)  :', f12.6)
 1150 format(1x, '                      b            (1/A)  :', f12.6)

 1160 format(1x, 'target sublimation energy           (eV)  :', f12.6)
 1170 format(1x, 'incident energy of gas ion          (eV)  :', f12.6)

 1200 format(1x, 'sputtering gas temperature           (K)  :', f12.6)
 1210 format(1x, 'sputtering gas pressure (gauge val) (Pa)  :', f12.6)
 1215 format(1x, '    assumed gas Temp. of ion gauge   (K)  :', f12.6)
 1217 format(1x, 'sputtering gas pressure (actual val)(Pa)  :', f12.6)
 1218 format(1x, 'sputtering gas density            (m^-3)  :', e12.6e2)
 1220 format(1x, 'maximum collision parameter          (m)  :', e12.6e2) 
 1230 format(1x, 'therm. threshold, unit T of kT       (K)  :', f12.6)

 1300 format(1x, 'Sca func node number in energy axis       :', i12) 
 1310 format(1x, '         node number in cp axis           :', i12) 
 1320 format(1x, '         maximum energy             (eV)  :', f12.6) 
 1330 format(1x, '         minimum energy             (eV)  :', f12.6) 
 1340 format(1x, '         maximum collisio param.     (A)  :', f12.6) 

 1400 format(1x, 'Col func node number in energy axis       :', i12) 
 1410 format(1x, '         node number in prob.  axis       :', i12) 
 1420 format(1x, '         maximum energy             (eV)  :', f12.6) 
 1430 format(1x, '         minimum energy             (eV)  :', f12.6) 
 1440 format(1x, '         maximum probability              :', f12.6)
 1450 format(1x, '         minimum probability              :', f12.6) 

 1500 format(1x, 'number of boundary elements               :', i12)
 1510 format(1x, ' node: ', i3, '(', f8.4, ',', f8.4, ')-(',
     $     f8.4, ',', f8.4, ') (m)')

 1600 format(1x, 'number of erosion track elements          :', i12) 
 1610 format(1x, ' node: ', i3, '(', f8.4, ',', f8.4, ')',
     $     ' depth :', f8.4, '(m)')

 1700 format(1x, 'times to take snapshots (s)')
 1710 format(1x, I3, ': ', g8.2)
 1720 format(1x, '== end of read data ==')

      write(6,1010) seed
      write(6,1020) ntrial

      write(6,1100) elmpar
      write(6,1110) mpar
      write(6,1120) elmgas
      write(6,1130) mgas
      write(6,1140) bmpca
      write(6,1150) bmpcb

      write(6,1160) eb
      write(6,1170) e1

      write(6,1200) tgas
      write(6,1210) pgasig
      write(6,1215) tgasig
      write(6,1217) pgas
      write(6,1218) ngas
      write(6,1220) rmax
      
      write(6,1300) nesca
      write(6,1310) nbsca
      write(6,1320) emaxsca
      write(6,1330) eminsca
      write(6,1340) bmaxsca

      write(6,1400) necol
      write(6,1410) nwcol
      write(6,1420) emaxcol
      write(6,1430) emincol
      write(6,1440) wmaxcol
      write(6,1450) wmincol

      write(6,1500) nnode
      do 110 inode = 1, nnode
         write(6,1510) inode, rnode(inode), znode(inode),
     $        rnode(inode+1), znode(inode+1)
 110  continue

      write(6,1600) npej
      do 120 ipej = 1, npej
         write(6,1610) ipej, rpej(ipej), zpej, dpej(ipej)
 120  continue

      write(6,1700)
      do 130 itime = 1, ntime
         write(6,1710) itime, tsnap(itime)
 130  continue
      write(6,1720)

      return
      end

