#!/usr/bin/perl
#
# $Id: fgraph.pl,v 1.10 2002/04/09 17:24:18 nakano Exp $
# visualize the result of pspread using MATLAB
#
# consts
$kb = 1.380658e-23;
$rvp = 10; # = vpmax / vpmean
$dispp = 5000;

if (@ARGV < 3) {die "$0 infile element elemchar [color] [prange] [vrange] [elevation] [azimuth]\n"};
$dfile = $ARGV[0];
$elem = $ARGV[1];
$spec = $ARGV[2];

if ($#ARGV > 2){$pcolor = $ARGV[3]}
else {$pcolor = "r"};

if ($#ARGV > 3){$prange = $ARGV[4]}
else {$prange = 20};

if ($#ARGV > 4){$vrange = $ARGV[5]}
else {$vrange = 5000};

if ($#ARGV > 5){$elevation = $ARGV[6]}
else {$elevation = 30};

if ($#ARGV > 6){$azimuth = $ARGV[7]}
else {$azimuth = -37.5};

open IF, "bzip2 -dc $dfile |" or die "cannot open $dfile";
#########################################################
# read particle mass and gas temp.
#
while(<IF>){if (/^ sputterd particle element/) {last};}
    $_ = <IF>;
    chomp;
    @line = split/:/;
    $mp = $line[1];

while(<IF>){
    if (/^ sputtering gas temperature/){
	chomp;
	@line = split/:/;
	$tg = $line[1];
	last;
    }
}
if ($tg > 0){
    $vpm = sqrt(2 * $kb * $tg / $mp);
} else {
    $vpm = sqrt(2 * $kb * 400 / $mp);
}

#########################################################
# read snapshot times
#
while(<IF>){
    if (/^ times to take snapshots \(s\)/) {last};
}

$it = 0;
while(<IF>){
    if (/== end of read data ==/) {last};
    chomp;
    @line = split(/: /);
    $snap[$it] = $line[1];
    $stidx{$line[1]} = $it;
    $it++;
}
$nt = $it;
close IF;

#########################################################
# parse data:
#
for($it = 0; $it < $nt; $it++){
    open GF, "bzip2 -dc $dfile | grep \'^$snap[$it]\' | head -n $dispp |" or die "cannot exec grep";
    $pp[$it] = "";
    $vp[$it] = "";
    while(<GF>){
	chomp;
	@line = split;
        $pp[$it] .= "$line[1] $line[2] $line[3];\n";
        $vp[$it] .= "$line[4] $line[5] $line[6];\n";
    }
    close GF;
}

for($it = 0; $it < $nt; $it++){
    open GF, "bzip2 -dc $dfile | grep \'^$snap[$it]\' |" or die "cannot exec grep";
    @{$sp[$it]} = ();
    while(<GF>){
	chomp;
	@line = split;
        push(@{$sp[$it]}, $line[7]);
    }
    close GF;
}

#########################################################
# speed distribution
#
$np = $#{$sp[0]};
$vpmin = 0;
$vpmax = $vpm * $rvp;
$vpstep = 30;
$nreg = int($vpmax/$vpstep);
#$nreg = int(sqrt($np));
#$vpstep = $vpmax / $nreg;

for($it = 0; $it < $nt; $it++){
    for($ireg=0;$ireg<$nreg;$ireg++){
	$freq[$it][$ireg] = 0;
    }
    while($svp = pop(@{$sp[$it]})){
	$ireg = ($svp - $vpmin) / $vpstep;
        $freq[$it][$ireg]++;
    }
    for($ireg=0;$ireg<$nreg;$ireg++){
	$freq[$it][$ireg] /= $vpstep * $np;
    }
}

open HF, "> hist-$spec.dat" or die "cannot open hist-$spec.dat";
print HF "# evolution of $elem particle speed histgram\n";
print HF "# width of each region is $vpstep (m/s)\n";
print HF "# Vp(m/s)\\t(us)\t";
for($it = 0; $it < $nt; $it++){
    printf HF "\t%9.3e", $snap[$it];
}
print HF "\n";

for($ireg = 0; $ireg < $nreg; $ireg++){
    $vps = ($ireg + 0.5) * $vpstep;
    printf HF "%9.3e", $vps;
    for($it = 0; $it < $nt; $it++){
	printf HF "\t%9.3e", $freq[$it][$ireg];
    }
    print HF "\n";
}
close HF;

#########################################################
# point dispersion
#
open DF, "> diff_$spec.m";
for($it=0;$it<$nt;$it++){
    print DF $diffhead;
    print DF $pp[$it];
    print DF $diffbody1;

    print DF "plot3(PX, PY, PZ, \'$pcolor.\', \'MarkerSize\', 2);\n";

    print DF $diffbody2;

    print DF "axis([-$prange $prange -$prange $prange -$prange $prange]);\n";
    print DF "tick = -$prange:$prange:$prange;\n";

    print DF $diffbody3;

    $tus = $snap[$it] * 1.0E6;
#    print DF "title(\'$elem: $tus \mus\', \'fontsize\', 30);\n";
    print DF "view([$azimuth $elevation]);\n";
    print DF "print -depsc2 diff$spec$it.eps\n";
#   print DF "pause\n";
}
close DF;

#########################################################
# velosity dispersion
#
open VF, "> velo_$spec.m";
for($it=0;$it<$nt;$it++){
    print VF $velohead;
    print VF $vp[$it];
    print VF $velobody1;

    print VF "plot3(VX, VY, VZ, \'$pcolor.\', \'MarkerSize\', 2);\n";

    print VF $velobody2;

    print VF "axis([-$vrange $vrange -$vrange $vrange -$vrange $vrange]);\n";
    print VF "tick = -$vrange:$vrange:$vrange;\n";

    print VF $velobody3;

    $tus = $snap[$it] * 1.0E6;
#    print VF "title(\'$elem: $tus \mus\', \'fontsize\', 30);\n";
    print VF "view([$azimuth $elevation]);\n";
    print VF "print -depsc2 velo$spec$it.eps\n";
#   print VF "pause\n";
}
close VF;


BEGIN{
$diffhead = << 'EOM';
clear P;
P = [
EOM

$diffbody1 = << 'EOM';
];

PX = P(:,1);
PY = P(:,2);
PZ = P(:,3);

hold off;
EOM

$diffbody2 = << 'EOM';
hold on;

set(gca, 'position', [0.15 0.13 0.7 0.7]);

axis equal;
EOM

$diffbody3 = << 'EOM';
set(findobj('type', 'axes'), 'linewidth', 3);
set(findobj('type', 'axes'), 'fontsize', 15);
grid on;
set(gca, 'xtick', tick);
set(gca, 'ytick', tick);
set(gca, 'ztick', tick);

xlabel('X (cm)', 'fontsize', 20);
ylabel('Y (cm)', 'fontsize', 20);
zlabel('Z (cm)', 'fontsize', 20);
EOM

$velohead = << 'EOM';
clear V;
V = [
EOM

$velobody1 = << 'EOM';
];

VX = V(:,1);
VY = V(:,2);
VZ = V(:,3);

hold off;
EOM

$velobody2 = << 'EOM';
hold on;

set(gca, 'position', [0.15 0.13 0.7 0.7]);

axis equal;
EOM

$velobody3 = << 'EOM';
set(findobj('type', 'axes'), 'linewidth', 3);
set(findobj('type', 'axes'), 'fontsize', 15);
grid on;
set(gca, 'xtick', tick);
set(gca, 'ytick', tick);
set(gca, 'ztick', tick);

xlabel('V_X (m/s)', 'fontsize', 20);
ylabel('V_Y (m/s)', 'fontsize', 20);
zlabel('V_Z (m/s)', 'fontsize', 20);
EOM
}
