c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998-2000 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: fmain.F,v 1.3 2001/07/24 09:23:43 nakano Exp $
c
#include "hisput.h"

      program spreadf
c
c     main routine of HISPUT
c
c     == params ==
c     BASE
c       seed   ... seed for random generator sgenrand
c       ntrial ... trial number
c
c     PHYSICAL PARAMETERS
c       tgas   ... tempereture of the gas     (K)
c       ngas   ... density of the gas         (m^-3)
c       mgas   ... mass of gas atom           (kg)
c       mpar   ...         sputtered particle (kg)
c       rmax   ... maximum paremeter of the col-param (m)
c       ttherm ... thermalization threshold, in unit T of kT (K)
c
c     EJECTION
c       e1     ... jection energy of particle (eV)
c
c     SCATTERING
c       POTENTIAL PARAMETERS (see sp.f for details)
c         bmpca  ... Born-Mayer Potential parameter, A (eV)
c         bmpcb  ... Born-Mayer Potential parameter, B (angstrom^-1)
c
c       SPLINE FUNCTION PARAMETERS
c         nesca   ... number of nodes along energy axis
c         nbsca   ...                       col-param axis
c         emaxsca ... maximum value of energy axis (eV)
c         eminsca ... minimum
c         bmaxsca ... maximum value of col-param axis (angstrom)
c
c     COLLISION
c       SPLINE FUNCTION PARAMETERS
c         necol   ... number of nodes along energy axis
c         nwcol   ...                       w(prob.)
c         emaxcol ... maximum of energy axis (eV)
c         emincol ... minimum
c         wmincol ... minimum of w axis, below which vg is prop. to w
c         wmaxcol ... maximun of w axis, above which vg is const.
c

      integer seed
      integer itrial, ntrial

      real*8 tgas, ngas
      real*8 mgas, mpar, rmax

      real*8 e1

      real*8 bmpca, bmpcb
      integer nesca, nbsca
      real*8 emaxsca, eminsca, bmaxsca

      integer necol, nwcol
      real*8 emaxcol, emincol, wmincol, wmaxcol

      real*8 genrand

      real*8 ep, vp(3), xp0(3), xp1(3), xsnap(3)
      real*8 vg, vr, fp, w

      real*8 unitch, emr2p
      parameter (unitch = UNITCH)
      real*8 vpar, vvp(3)

      integer itime, ntime, ntimemax
      parameter (ntimemax = 10)
      real*8 tpar, tpath, tsnap(ntimemax)
      real*8 fpsnap

      integer i

c
c     start:
c
      call readin(seed, ntrial,
     $     bmpca, bmpcb,
     $     nesca, nbsca, emaxsca, eminsca, bmaxsca,
     $     tgas, ngas, mgas, mpar, rmax,
     $     necol, nwcol, emaxcol, emincol, wmaxcol, wmincol, e1,
     $     ntimemax, ntime, tsnap)

      call sgenrand(seed)
      call inispf1
      call inispf2
      call hiab

      call setsp(bmpca, bmpcb)
      call setsca(nesca, nbsca, emaxsca, eminsca, bmaxsca)
      call setcol(tgas, ngas, mgas, mpar, rmax,
     $     necol, nwcol, emaxcol, emincol, wmaxcol, wmincol)

      call setcdir(mpar, mgas, rmax)

      emr2p = 2 * unitch / mpar

      write(6,1000)
 1000 format('TIME(s) POS(cm)',23x,'VELO(m/s)',21x,'V(m/s)')

      do 10 itrial = 1, ntrial

         ep = e1
         tpar = 0.0d0
         itime = 1

         vp(1) = 0.0d0
         vp(2) = 0.0d0
         vp(3) = 1.0d0

         xp0(1) = 0.0d0
         xp0(2) = 0.0d0
         xp0(3) = 0.0d0

 25      continue

c
c     -- particle path --
c
         w = genrand()
         call frpath(ep, w, fp)

         vpar = sqrt(ep*emr2p)
         tpath = fp/vpar
         if (tpar + tpath .ge. tsnap(itime)) then
            fpsnap = fp * (tsnap(itime) - tpar) / tpath

            do 27 i = 1, 3
               xsnap(i) = xp0(i) + fpsnap * vp(i)
               vvp(i) = vpar * vp(i)
 27         continue
C
C print out in cm order
C
            write(6,1010) tsnap(itime), (xsnap(i)*100,i=1,3),
     $                  (vvp(i),i=1,3), vpar
 1010       format(g8.2, 3f10.4, 3f10.2, g10.3)
            itime = itime + 1
            if (itime .gt. ntime) go to 10
         end if
         tpar = tpar + fp/vpar


         do 30 i = 1, 3
            xp1(i) = xp0(i) + fp * vp(i)
 30      continue

         do 32 i = 1, 3
            xp0(i) = xp1(i)
 32      continue

c
c     -- collision --
c
         w = genrand()
         call colgas(ep, w, vg)
         w = genrand()
         call relv(ep, vg, w, vr)
         call cdir(ep, vp, vg, vr)

         go to 25
C
C        WRITE result
C
 10   continue

      stop
      end

