#!/usr/bin/perl
#
# $Id: pgraph.pl,v 1.5 2001/07/24 09:23:43 nakano Exp $
# visualize the result of pspread using gnuplot
#
# SELECT DEVICE
$dev = "ps"; # or "tek", if tek40xx is used
$eu    = 3000.0;
$estep = 30;

if (@ARGV < 3) {die "$0 infile element elemchar [ps]\n"};
$dfile = $ARGV[0];
$elem = $ARGV[1];
$spec = $ARGV[2];

if ($ARG[3] eq "ps"){
    $ext = "eps";
    $gphead = "set terminal postscript eps plus";
    $gptitl = "set title \'\\huge ";
} else {
    $ext = "tek";
    $gphead = "set terminal tek40xx";
    $gptitl = "set title \'";
}

open IF, "$dfile" or die "cannot open $dfile";
while(<IF>){
    if (/^ times to take snapshots \(s\)/) {last};
}

$i = 1;
while(<IF>){
    if (/== end of read data ==/) {last};
    chomp;
    @line = split(/: /);
    $snap[$i] = $line[1];
    $time_ns[$i] = $snap[$i] * 1e6;
    $i++;
}
close IF;

#
# energy distribution
#
open IF, "$dfile";
while(<IF>){
    chomp;
    for($i=1;$i<=$#snap;$i++){
        unless(/^$snap[$i]/){next;}
        @line = split;
        $intvl = int($line[7] / $estep)+1;
        ${freq[$i]}[$intvl]++;
        $total[$i]++;
    }
}
close IF;

open HF, "> hist$spec.dat" or die "cannot open hist$spec.dat";

print HF "#E(eV)\\t(ns)";
for($i=1;$i<=$#snap;$i++){
    print HF "\t$time_ns[$i]";
}
print HF "\n";

for($intvl=1;$intvl<=int($eu/$estep);$intvl++){
    printf HF "%5.2f", $intvl * $estep;
    for($i=1;$i<=$#snap;$i++){
	printf HF "\t%10.4e", ${freq[$i]}[$intvl] / $total[$i] / $estep;
    }
    print HF "\n";
}
close HF;

for ($i=1;$i<=$#snap;$i++){
#   system("grep \'^$snap[$i]\' $dfile > pdata$spec$i.dat");

#
# point dispersion
#
    open GP, "> diff$spec$i.gp";
    print GP "$gphead\n";
    print GP "set output \"diff$spec$i.$ext\"\n";
    print GP "${gptitl}$elem $time_ns[$i] ns\'\n";
    print GP "$diffbody";
    print GP "splot \"< grep \'^$snap[$i]\' $dfile\" using 2:3:4 with dots\n";
    close GP;

    system("gnuplot diff$spec$i.gp");

#
# velosity dispersion
#
    open GP, "> velo$spec$i.gp";
    print GP "$gphead\n";
    print GP "set output \"velo$spec$i.$ext\"\n";
    print GP "${gptitl}$elem $time_ns[$i] ns\'\n";
    print GP "$velobody";
    print GP "splot \"< grep \'^$snap[$i]\' $dfile\" using 5:6:7 with dots\n";
#    print GP "splot \"pdata$spec$i.dat\" using 5:6:7 with dots\n";
    close GP;

    system("gnuplot velo$spec$i.gp");
}


#set terminal postscript eps plus
#set output "diffB2.tk"
#set title "B 0.1s" 0,-2
#"set title '\huge B 0.1s' 0,-2
#splot "pout.plot" using 2:3:4 with dots 

BEGIN{
    $diffbody = << 'EOM';
set dummy u,v
set format x "%g"
set format y "%g"
set format z "%g"
set parametric
set view 60, 30, 1, 1
set size 0.7,1
set data style points
set tics in
set ticslevel 0
set xtics -4, 2, 4
set ytics -4, 2, 4
set ztics -4, 2, 4
set xrange [-5 : 5]
set yrange [-5 : 5]
set zrange [-5 : 5]
set xlabel "x(cm)" -2,-1
set ylabel "y(cm)"  2,-1
set label 1 "z(cm)" at -5,-5, 7 center
set noautoscale xy
set noautoscale z
set zero 1e-08
set nokey
EOM

    $velobody = << 'EOM';
set dummy u,v
set format x "%g"
set format y "%g"
set format z "%g"
set parametric
set view 60, 30, 1, 1
set size 0.7,1
set data style points
set tics in
set ticslevel 0
set xtics -1000,1000,1000
set ytics -1000,1000,1000
set ztics -1000,1000,1000
set xrange [-2000 : 2000]
set yrange [-2000 : 2000]
set zrange [-2000 : 2000]
set xlabel "Vx(m/s)" -2,-1
set ylabel "Vy(m/s)" 2,-2
set label 1 "Vz(m/s)" at -2000,-2000, 3000 center
set noautoscale xy
set noautoscale z
set zero 1e-08
set nokey
EOM
}
