#!/usr/bin/perl
# $Id: 2dmap.pl,v 1.2 2001/10/22 21:44:20 nakano Exp $
# particle density summarizing script.
#
use PGPLOT;
use Getopt::Long;

GetOptions(
	"scale" => \$scale,
	"omitx" => \$omitx,
	"omity" => \$omity
);

$dmin = 0;

($vzmin, $vzmax, $vzofs, $vrmax, $vstep, $dmax, $file)=@ARGV;
if (($vzmin >= $vzmax) || ($vrmax <= 0) || ($dmax <= 0) || ($file eq undef)){
	print STDERR "$0 vzmin vzmax vzofs vrmax vstep dmax file\n";
	die;
}

$nz = int(($vzmax - $vzmin) / $vstep);
$nr = int($vrmax / $vstep);

for($iz=0;$iz<$nz;$iz++){
    for($ir=0;$ir<$nr;$ir++){
	$count[$ir][$iz] = 0;
    }
}

open DF, "$file" or die "cannot open $file";
$np = 0;
while(<DF>){
    chomp;
    @line = split;
    $vr = sqrt($line[0]**2 + $line[1]**2);
    $vz = @line[2];

    $ir = int($vr / $vstep);
    $iz = int(($vz - $vzmin + $vzofs) / $vstep);

    if ($ir < $nr && $iz >= 0 && $iz < $nz) {
	$count[$ir][$iz]++;
    }
    $np++;
}
close DF;

for($iz=0;$iz<$nz;$iz++){
    for($ir=0;$ir<$nr;$ir++){
	$dens[$ir][$iz] = 1.0 / $np * $count[$ir][$iz]
		/ ($vstep**2 * 6.28 * (2 * $ir + 1));
    }
}

pgopen("$file.ps/CPS");
#pgsci(1);

# examples from pgdemo4.f in /usr/doc/pgplot5/examples
#
#        -- gray scale
#      DATA GL /0.0, 1.0/
#      DATA GR /0.0, 1.0/
#      DATA GG /0.0, 1.0/
#      DATA GB /0.0, 1.0/
#pgctab(\@l, \@r, \@g, \@b, 2, 1, 0.5);

#        -- rainbow
#      DATA RL /-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
#      DATA RR / 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
#      DATA RG / 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
#      DATA RB / 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/
#
@l= (-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7);
@r= ( 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0);
@g= ( 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0);
@b= ( 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0);
pgctab(\@l, \@r, \@g, \@b, 9, 1, 0.5);

#        -- heat
#      DATA HL /0.0, 0.2, 0.4, 0.6, 1.0/
#      DATA HR /0.0, 0.5, 1.0, 1.0, 1.0/
#      DATA HG /0.0, 0.0, 0.5, 1.0, 1.0/
#      DATA HB /0.0, 0.0, 0.0, 0.3, 1.0/
#
#@l=(0.0, 0.2, 0.4, 0.6, 1.0);
#@r=(0.0, 0.5, 1.0, 1.0, 1.0);
#@g=(0.0, 0.0, 0.5, 1.0, 1.0);
#@b=(0.0, 0.0, 0.0, 0.3, 1.0);
#pgctab(\@l, \@r, \@g, \@b, 5, 1, 0.5);

pgenv($vzmin, $vzmax, 0, $vrmax, 1, -2); 

if ($omitx eq undef){$xopt = "N"}
if ($omity eq undef){$yopt = "N"}
pgbox($xopt, 1000, 1, $yopt, 1000, 1);

#
# draw a wedge.
#
if ($scale ne undef){
    $wsize = 3.0;
    $wdisp = 1.0;
    $wwidth = 1.0;
    pgsch($wsize);
    pgsci(1);
    pgwedg('TI', $wdisp, $wwidth, $dmin, $dmax, ' ');
}

#
# density image
#
#for($iz=0;$iz<$nz;$iz++){
#    for($ir=0;$ir<$nr;$ir++){
#        print "$dens[$ir][$iz]\n";
#    }
#    print "\n";
#}

@tr = ($vzmin-$vstep/2, $vstep, 0, -$vstep/2, 0, $vstep);
pgimag(\@dens, $nz, $nr, 1, $nz, 1, $nr, $dmin, $dmax, \@tr);

#pgsch($tsize); pgscf($tfont);
#pgmtxt('B', $tdisp, $tpos, $tcord, '2 Pa');
#pgsch(1);

pgend;

