c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: spline1d.F,v 1.3 2000/11/19 20:17:38 nakano Exp $
c
c     1 dimensional spline function.  Multile functions can be
c     used simultaneously (select with idx)
c
c     common block /cspf1/
c       resv(idx) .. wheather idx is reserved (0:open, 1:reserved)
c       nq(idx)   .. number of node points. (ndim + ks)
c       kq(idx)   .. ks reserver
c       qs        .. node points. (ndim + ks)
c       a         .. parameter
c
#include "spline.h"

      subroutine inispf1
c
c     initializing common block variables
c
      integer idxmax, nsmax, ksmax
      parameter(idxmax = ISP1MAX)
      parameter(nsmax = NMAX, ksmax = KMAX)

      integer resv(idxmax)
      integer nsq(idxmax), ksq(idxmax)
      real*8 qs(idxmax, nsmax+ksmax)
      real*8 a(idxmax, nsmax)
      common /cspf1/ resv, nsq, ksq, qs, a

      integer i, j

      do 10 i = 1, idxmax
         resv(i) = 0
         nsq(i) = 0
         ksq(i) = 0
         do 20 j = 1, nsmax+ksmax
            qs(i,j) = 0
 20      continue
 10   continue

      return
      end


      subroutine setspf1(idx, ns, ks, x, v)
c
c     create the matrix of N x N dimension for the determination of
c     parameters of spline function order ICHI using Deboor Cox argorism
c
c     assumption.. the data point is placed periodically
c     (i.e. x[j] = ja)
c
c     input
c       idx  .. index of spline function
c       n    .. number of reference points
c       ks   .. order of spline function
c       x(n) .. reference point. if j>i, x(j) > x(i).
c       v(n) .. func value at x(i)
c
      integer ns, ks
      real*8 x(ns), v(ns)

      integer idxmax, nsmax, ksmax
      parameter(idxmax = ISP1MAX)
      parameter(nsmax = NMAX, ksmax = KMAX)

      integer resv(idxmax)
      integer nsq(idxmax), ksq(idxmax)
      real*8 qs(idxmax, nsmax+ksmax)
      real*8 a(idxmax, nsmax)
      common /cspf1/ resv, nsq, ksq, qs, a

      real*8 bi(nsmax+1), bf, bl
      real*8 at(nsmax)
      integer i, j, k

c     for sparse library
c
      integer matrix, error, sfCreate, sfGetElement, sfFactor
      integer ielm

c     condition check
c
      if (ns .gt. nsmax) then
         write(*,1000) 'ns', ns
         stop
      end if
      if (ks .gt. ksmax) then
         write(*,1000) 'ks', ks
         stop
      end if
 1000 format(1x, '(pspf) parameter out of range : ', A, I5)

      if (resv(idx) .eq. 1) then
         write(*,1010) idx
         stop
      end if
 1010 format(1x, '(setspf1) index ', I2, ' has been already reserved.')

      resv(idx) = 1
      nsq(idx) = ns
      ksq(idx) = ks

      do 10 j=1, ks
         qs(idx,j) = x(1)
 10   continue

      do 20 j=1, ns-ks
         qs(idx,j+ks) = 0.5d0 * (x(j) + x(j+ks))
 20   continue

      do 30 j=ns+1, ns+ks
         qs(idx,j) = x(ns)
 30   continue

#ifdef DEBUG
      do 35 j = 1, ns+ks
         write (0,9000) j, qs(idx,j)
   35 continue
 9000 format(I3, ":", F12.6)
#endif

c     initialize sparse matrix
c
      matrix = sfCreate(ns, 0, error)
      call sfClear(matrix)

      do 100 i=1, ns
         do 50 j = 1, ns-1
            if (x(i).ge.qs(idx,j) .and. x(i).lt.qs(idx,j+1)) then
               bi(j) = 1.0d0
            else
               bi(j) = 0.0d0
            end if
 50      continue

         do 55 j = ns, ns+1
            if (x(i).ge.qs(idx,j) .and. x(i).le.qs(idx,j+1)) then
               bi(j) = 1.0d0
            else
               bi(j) = 0.0d0
            end if
 55      continue

         do 80 k = 2, ks
            do 70 j = 1, ns
               if (qs(idx,j) .lt. qs(idx,j+k-1)) then
                  bf = (x(i) - qs(idx,j)) /
     $                 (qs(idx,j+k-1) - qs(idx,j)) * bi(j)
               else
                  bf = 0.0d0
               end if

               if (qs(idx,j+1) .lt. qs(idx,j+k)) then
                  bl = (qs(idx,j+k) - x(i)) /
     $                 (qs(idx,j+k) - qs(idx,j+1)) * bi(j+1)
               else
                  bl = 0.0d0
               end if

               bi(j) = bf + bl
 70         continue
 80      continue

         do 90 j=1, ns
            if (bi(j) .ne. 0.0d0) then
               ielm = sfGetElement(matrix, i, j)
               call sfAdd1Real(ielm, bi(j))
            end if
 90      continue

 100  continue

c     solve matrix
c
#ifdef DEBUG
      call sfPrint(matrix, .true., .true., .true.)
#endif
      error = sfFactor(matrix)
      call sfSolve(matrix, v, at)

c     free'ing of the matrix may lead to the seg. fault...
c
c     call sfDestroy(matrix)

      do 110 i = 1, ns
         a(idx, i) = at(i)
 110  continue

      return
      end


c-------------------------------
      double precision function spf1 (idx, x)
c
c     calculate the value of 1D spline function at (x).
c
c     input
c       idx .. index of spline function
c       x   .. input data
c
      integer idx
      real*8 x

      integer idxmax, nsmax, ksmax
      parameter(idxmax = ISP1MAX)
      parameter(nsmax = NMAX, ksmax = KMAX)

      integer resv(idxmax)
      integer nsq(idxmax), ksq(idxmax)
      real*8 qs(idxmax, nsmax+ksmax)
      real*8 a(idxmax, nsmax)
      common /cspf1/ resv, nsq, ksq, qs, a

      integer ns, ks
      real*8 bi(nsmax+1), bf, bl
      integer j, k, is
      real*8 s

      if (idx .lt. 1 .or. idx .gt. idxmax) then
         write(*,1010) idx
         stop
      end if
 1010 format(1x,'(spf1)-- index ', I2, ' is out of range')

      if (resv(idx) .ne. 1) then
         write(*,1020) idx
         stop
      end if
 1020 format(1x,'(spf1)-- parameter for index ', I2, ' is not set')

      ns = nsq(idx)
      ks = ksq(idx)

      if (x .lt. qs(idx,ks) .or. x .gt. qs(idx,ns+ks)) then
         write (*, 1000) x
 1000    format(' (spf1) -- parameter out of range: x ', 1f13.5)
         stop
      end if

      do 10 is = ks, ns-1
         if (x .lt. qs(idx,is+1)) go to 11
 10   continue
      is = ns
 11   continue

c
c     B-spline
c
      do 50 j = is-ks+1, min(ns+1, is+ks-1)
         if (x .lt. qs(idx,j) .or. x .gt. qs(idx,j+1)) then
            bi(j) = 0.0d0
         else if (j .lt. ns .and. x .eq. qs(idx,j+1)) then
            bi(j) = 0.0d0
         else 
            bi(j) = 1.0d0
         end if
 50   continue
      
      do 80 k = 2, ks
         do 70 j = is-ks+1, min(ns, is+ks-k)
            if (qs(idx,j) .lt. qs(idx,j+k-1)) then
               bf = (x - qs(idx,j)) /
     $              (qs(idx,j+k-1) - qs(idx,j)) * bi(j)
            else
               bf = 0.0d0
            end if
            
            if (qs(idx,j+1) .lt. qs(idx,j+k)) then
               bl = (qs(idx,j+k) - x) /
     $              (qs(idx,j+k) - qs(idx,j+1)) * bi(j+1)
            else
               bl = 0.0d0
            end if
            
            bi(j) = bf + bl
            
 70      continue
 80   continue

      s = 0.0d0
      do 90 j = is-ks+1, is
         s = s + bi(j) * a(idx,j)
 90   continue
      
      spf1 = s

      return
      end

c-------------------------------
      double precision function dspf1 (idx, x)
c
c     calculate the derivative of 1D spline function at (x).
c
c     input
c       idx .. index of spline function
c       x   .. input data
c
      integer idx
      real*8 x

      integer idxmax, nsmax, ksmax
      parameter(idxmax = ISP1MAX)
      parameter(nsmax = NMAX, ksmax = KMAX)

      integer resv(idxmax)
      integer nsq(idxmax), ksq(idxmax)
      real*8 qs(idxmax, nsmax+ksmax)
      real*8 a(idxmax, nsmax)
      common /cspf1/ resv, nsq, ksq, qs, a

      integer ns, ks
      real*8 bi(nsmax+1), bf, bl
      integer j, k, is
      real*8 s

      if (idx .lt. 1 .or. idx .gt. idxmax) then
         write(*,1010) idx
         stop
      end if
 1010 format(1x,'(dspf1)-- index ', I2, ' is out of range')

      if (resv(idx) .ne. 1) then
         write(*,1020) idx
         stop
      end if
 1020 format(1x,'(dspf1)-- parameter for index ', I2, ' is not set')

      ns = nsq(idx)
      ks = ksq(idx)

      if (x .lt. qs(idx,ks) .or. x .gt. qs(idx,ns+ks)) then
         write (*, 1000) x
 1000    format(' (dspf1) -- parameter out of range: x ', 1f13.5)
         stop
      end if

c
c     ks == 1, dspf1 = 0
c
      if (ks .eq. 1) then
         dspf1 = 0.0d0
         return
      end if

      do 10 is = ks, ns-1
         if (x .lt. qs(idx,is+1)) go to 11
 10   continue
      is = ns
 11   continue

c
c     B-spline, first order
c
      do 50 j = is-ks+1, min(ns+1, is+ks-1)
         if (x .lt. qs(idx,j) .or. x .gt. qs(idx,j+1)) then
            bi(j) = 0.0d0
         else if (j .lt. ns .and. x .eq. qs(idx,j+1)) then
            bi(j) = 0.0d0
         else 
            bi(j) = 1.0d0
         end if
 50   continue
      
      do 80 k = 2, ks-1
         do 70 j = is-ks+1, min(ns, is+ks-k)
            if (qs(idx,j) .lt. qs(idx,j+k-1)) then
               bf = (x - qs(idx,j)) /
     $              (qs(idx,j+k-1) - qs(idx,j)) * bi(j)
            else
               bf = 0.0d0
            end if
            
            if (qs(idx,j+1) .lt. qs(idx,j+k)) then
               bl = (qs(idx,j+k) - x) /
     $              (qs(idx,j+k) - qs(idx,j+1)) * bi(j+1)
            else
               bl = 0.0d0
            end if
            
            bi(j) = bf + bl
            
 70      continue
 80   continue

      s = 0.0d0
      do 90 j = is-ks+1, is
         if (qs(idx, j+ks-1) .ne. qs(idx, j)) then
            s = s + a(idx,j) * bi(j) / (qs(idx, j+ks-1) - qs(idx, j))
         end if

         if (qs(idx, j+ks) .ne. qs(idx, j+1)) then
            s = s - a(idx,j) * bi(j+1) / (qs(idx, j+ks) - qs(idx, j+1))
         end if
 90   continue
      
      dspf1 = (ks - 1) * s

      return
      end
