c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: spline2d.F,v 1.5 2001/01/13 07:35:06 nakano Exp $
c
c     2 dimensional spline function.  Multile functions can be
c     used simultaneously (select with idx)
c
c     common block cspf2
c       resv     .. whether index is reserved (0:open, 1:reserved)
c       nxq      .. storage of nx
c       nyq      ..            ny
c       kxq      ..            kx
c       kyq      ..            ky
c       qx       .. node point along x. (nx+kx)-th vector.
c       qy       ..                  y. (ny+ky)
c       a        .. parameter vector.
c
#include "spline.h"

      subroutine inispf2
c
c     initializing common block variables
c
      integer nxmax, nymax, kxmax, kymax
      parameter(idxmax = ISP2MAX)
      parameter(nxmax = NMAX, nymax = NMAX)
      parameter(kxmax = KMAX, kymax = KMAX)

      integer resv(idxmax)
      integer nxq(idxmax), nyq(idxmax), kxq(idxmax), kyq(idxmax)
      real*8 qx(idxmax, nxmax+kxmax), qy(idxmax, nymax+kymax)
      real*8 a(idxmax, nxmax*nymax)
      common /cspf2/ resv, nxq, nyq, kxq, kyq, qx, qy, a

      integer i, j

      do 10 i = 1, idxmax
         resv(i) = 0
         nxq(i) = 0
         nyq(i) = 0
         kxq(i) = 0
         kyq(i) = 0
         do 20 j = 1, nxmax+kxmax
            qx(i,j) = 0
 20      continue
         do 30 j = 1, nymax+kymax
            qy(i,j) = 0
 30      continue
         do 40 j = 1, nxmax*nymax
            a(i,j) = 0
 40      continue
 10   continue

      return
      end

c-------------------------------------
      subroutine setspf2(idx, ndim, nx, ny, kx, ky, x, y, v)
c
c     prepare parameter matrix for 2D spline function
c
c     input
c       idx      .. index of spline function
c       ndim     .. declared row dimension of v in calling program
c       nx       .. number of data points along x axis
c       ny       ..                             y
c       kx       .. order of spline function for x axis
c       ky       ..                              y
c       x(nx)    .. data points along x axis
c       y(ny)    .. data points along y axis
c       v(nx,ny) .. func val at (x(i),y(j))
c
      integer idx, ndim, nx, ny, kx, ky
      real*8 x(nx), y(ny), v(ndim, ny)

      integer nxmax, nymax, kxmax, kymax
      parameter(idxmax = ISP2MAX)
      parameter(nxmax = NMAX, nymax = NMAX)
      parameter(kxmax = KMAX, kymax = KMAX)

      real*8 bx(nxmax, nxmax), by(nymax, nymax)
      real*8 bi(nxmax+1), b(nxmax*nymax), at(nxmax*nymax)
      integer i, j, k, l
      real*8 bf, bl

c     for sparse library
c
      integer matrix, error, sfCreate, sfGetElement, sfFactor
      integer ielm

c     common block
c
      integer resv(idxmax)
      integer nxq(idxmax), nyq(idxmax), kxq(idxmax), kyq(idxmax)
      real*8 qx(idxmax, nxmax+kxmax), qy(idxmax, nymax+kymax)
      real*8 a(idxmax, nxmax*nymax)
      common /cspf2/ resv, nxq, nyq, kxq, kyq, qx, qy, a

c     condition check
c
      if (idx .lt. 1 .or. idx .gt. idxmax) then
         write(*,1000) 'idx', idx
         stop
      end if
      if (nx .gt. nxmax) then
         write(*,1000) 'nx', nxmax
         stop
      end if
      if (kx .gt. kxmax) then
         write(*,1000) 'kx', kx
         stop
      end if
      if (ny .gt. nymax) then
         write(*,1000) 'ny', nymax
         stop
      end if
      if (ky .gt. kymax) then
         write(*,1000) 'ky', ky
         stop
      end if
 1000 format(1x, '(setspf2) parameter out of range : ', A, I5)

      if (resv(idx) .eq. 1) then
         write(*,1010) idx
         stop
      end if
 1010 format(1x, '(setspf2) index ', I2, ' has been already reserved.')

      resv(idx) = 1
      nxq(idx) = nx
      nyq(idx) = ny
      kxq(idx) = kx
      kyq(idx) = ky
c
c  X component
c
      do 10 j=1, kx
         qx(idx,j) = x(1)
 10   continue

      do 20 j=1, nx-kx
         qx(idx,j+kx) = 0.5d0 * (x(j) + x(j+kx))
 20   continue

      do 30 j=nx+1, nx+kx
         qx(idx,j) = x(nx)
 30   continue

      do 100 i=1, nx
         do 50 j = 1, nx-1
            if (x(i).ge.qx(idx,j) .and. x(i).lt.qx(idx,j+1)) then
               bi(j) = 1.0d0
            else
               bi(j) = 0.0d0
            end if
 50      continue

         do 55 j = nx, nx+1
            if (x(i).ge.qx(idx,j) .and. x(i).le.qx(idx,j+1)) then
               bi(j) = 1.0d0
            else
               bi(j) = 0.0d0
            end if
 55      continue

         do 80 k = 2, kx
            do 70 j = 1, nx
               if (qx(idx,j) .lt. qx(idx,j+k-1)) then
                  bf = (x(i)-qx(idx,j)) /
     $                 (qx(idx,j+k-1)-qx(idx,j)) * bi(j)
               else
                  bf = 0.0d0
               end if

               if (qx(idx,j+1) .lt. qx(idx,j+k)) then
                  bl = (qx(idx,j+k) - x(i)) /
     $                 (qx(idx,j+k) - qx(idx,j+1)) * bi(j+1)
               else
                  bl = 0.0d0
               end if

               bi(j) = bf + bl
 70         continue
 80      continue

         do 90 j=1,nx
            bx(i,j) = bi(j)
 90      continue

 100  continue

c
c  Y component
c
      do 110 j=1, ky 
         qy(idx,j) = y(1)
 110  continue

      do 120 j=1, ny-ky
         qy(idx,j+ky) = 0.5d0 * (y(j) + y(j+ky))
 120  continue
         
      do 130 j=ny+1, ny+ky
         qy(idx,j) = y(ny)
 130  continue

      do 200 i=1, ny
         do 150 j = 1, ny-1
            if (y(i).ge.qy(idx,j) .and. y(i).lt.qy(idx,j+1)) then
               bi(j) = 1.0d0
            else
               bi(j) = 0.0d0
            end if
 150      continue

         do 155 j = ny, ny+1
            if (y(i).ge.qy(idx,j) .and. y(i).le.qy(idx,j+1)) then
               bi(j) = 1.0d0
            else
               bi(j) = 0.0d0
            end if
 155     continue

         do 180 k = 2, ky
            do 170 j = 1, ny
               if (qy(idx,j) .lt. qy(idx,j+k-1)) then
                  bf = (y(i)-qy(idx,j)) /
     $                 (qy(idx,j+k-1)-qy(idx,j)) * bi(j)
               else
                  bf = 0.0d0
               end if

               if (qy(idx,j+1) .lt. qy(idx,j+k)) then
                  bl = (qy(idx,j+k) - y(i))/
     $                 (qy(idx,j+k) - qy(idx,j+1)) * bi(j+1)
               else
                  bl = 0.0d0
               end if

               bi(j) = bf + bl
 170        continue
 180     continue

         do 190 j=1,ny
            by(i,j) = bi(j)
 190     continue

 200  continue

c
c     solve matrix using sparse lib.
c
      matrix = sfCreate(nx*ny, 0, error)
      call sfClear(matrix)

      do 300 i=1, nx
         do 300 j=1, ny
            do 300 k = 1, nx
               do 300 l = 1, ny
                  if (bx(i,k) .ne. 0.0d0 .and. by(j,l) .ne. 0.0d0) then
                     ielm = sfGetElement(matrix, ny*(i-1)+j, ny*(k-1)+l)
                     call sfAdd1Real(ielm, bx(i,k)*by(j,l))
                  end if
 300  continue
       
      do 310 i = 1, nx
         do 310 j = 1, ny
            b(ny*(i-1)+j) = v(i,j)
 310  continue


#ifdef DEBUG
      call sfPrint(matrix, .false., .false., .false.)

      write(*,1001)(b(i),i=1,nx*ny)
 1001 format(16f6.2)
#endif
c
c     solve matrix
c
      error = sfFactor(matrix)
      call sfSolve(matrix, b, at)
c
c it causes segmentation fault somehow..
c
c      call sfDestroy(matrix)

      do 320 i = 1, nx*ny
         a(idx,i) = at(i)
 320  continue
#ifdef DEBUG
      write (*,2010) (a(idx,i),i=1,nx*ny)
 2010 format (' solution a(i)' /(1x, 5f9.5))
#endif

      return
      end


c-------------------------------------
      double precision function spf2(idx, xp, yp)
c
c     calculate the value of 2D spline function at (xp, yp).
c
      real*8 xp, yp

      integer nxmax, nymax, kxmax, kymax
      parameter(idxmax = ISP2MAX)
      parameter(nxmax = NMAX, nymax = NMAX)
      parameter(kxmax = KMAX, kymax = KMAX)

      real*8 bjx(nxmax+1), bjy(nymax+1)
      integer i, k, ix, iy
      real*8 bf, bl
      real*8 s

c     common block
c
      integer resv(idxmax)
      integer nxq(idxmax), nyq(idxmax), kxq(idxmax), kyq(idxmax)
      real*8 qx(idxmax, nxmax+kxmax), qy(idxmax, nymax+kymax)
      real*8 a(idxmax, nxmax*nymax)
      common /cspf2/ resv, nxq, nyq, kxq, kyq, qx, qy, a

      integer nx, ny, kx, ky

      if (idx .lt. 1 .or. idx .gt. idxmax) then
         write(*,1010) idx
         stop
      end if
 1010 format(1x,'(spf2)-- index ', I2, ' is out of range')

      if (resv(idx) .ne. 1) then
         write(*,1020) idx
         stop
      end if
 1020 format(1x,'(spf2)-- parameter for index ', I2, ' is not set') 

      nx = nxq(idx)
      ny = nyq(idx)
      kx = kxq(idx)
      ky = kyq(idx)

      if (xp .lt. qx(idx,kx) .or. xp .gt. qx(idx,nx+1)) then
         write (*, 1000) 'xp', xp, qx(idx,kx), qx(idx,nx+1)
         stop
      end if
      if (yp .lt. qy(idx,ky) .or. yp .gt. qy(idx,ny+1)) then
         write (*, 1000) 'yp', yp, qy(idx,ky), qy(idx,ny+1)
         stop
      end if
 1000 format(1x,'(spf2)-- parameter out of region: ', A, 3e13.5)

c
c     define the ix where the qx(idx,ix) <= xp < qx(idx,ix+1)
c
      do 10 ix = kx, nx-1
         if (xp .lt. qx(idx,ix+1)) go to 11
 10   continue
      ix = nx
 11   continue

      do 20 iy = ky, ny-1
         if (yp .lt. qy(idx,iy+1)) go to 21
 20   continue
      iy = ny
 21   continue

c
c     B-spline , X component
c
      do 50 i = ix-kx+1, min(nx+1, ix+kx-1)
         if (xp .lt. qx(idx,i) .or. xp .gt. qx(idx,i+1)) then
            bjx(i) = 0.0d0
         else if (i .lt. nx .and. xp .eq. qx(idx,i+1)) then
            bjx(i) = 0.0d0
         else 
            bjx(i) = 1.0d0
         end if
 50   continue
      
      do 80 k = 2, kx
         do 70 i = ix-kx+1, min(nx, ix+kx-k)
            if (qx(idx,i) .lt. qx(idx,i+k-1)) then
               bf = (xp-qx(idx,i)) / (qx(idx,i+k-1)-qx(idx,i)) * bjx(i)
            else
               bf = 0.0d0
            end if
            
            if (qx(idx,i+1) .lt. qx(idx,i+k)) then
               bl = (qx(idx,i+k) - xp) /
     $              (qx(idx,i+k) - qx(idx,i+1)) * bjx(i+1)
            else
               bl = 0.0d0
            end if
            
            bjx(i) = bf + bl

 70      continue
 80   continue

c
c     B-spline , Y component
c
      do 150 i = iy-ky+1, min(ny+1, iy+ky-1)
         if (yp .lt. qy(idx,i) .or. yp .gt. qy(idx,i+1)) then
            bjy(i) = 0.0d0
         else if (i .lt. ny .and. yp .eq. qy(idx,i+1)) then
            bjy(i) = 0.0d0
         else 
            bjy(i) = 1.0d0
         end if
 150  continue
      
      do 180 k = 2, ky
         do 170 i = iy-ky+1, min(ny, iy+ky-k)
            if (qy(idx,i) .lt. qy(idx,i+k-1)) then
               bf = (yp - qy(idx,i)) /
     $              (qy(idx,i+k-1) - qy(idx,i)) * bjy(i)
            else
               bf = 0.0d0
            end if
            
            if (qy(idx,i+1) .lt. qy(idx,i+k)) then
               bl = (qy(idx,i+k) - yp) /
     $              (qy(idx,i+k) - qy(idx,i+1)) * bjy(i+1)
            else
               bl = 0.0d0
            end if
            
            bjy(i) = bf + bl

 170     continue
 180  continue

      s = 0.0d0
      do 90 i = ix-kx+1, ix
         do 90 j = iy-ky+1, iy 
c            write(9,1100) i, j, bjx(i), bjy(j)
 1100       format(1x, 2i3, 2f13.6)
            s = s + bjx(i) * bjy(j) * a(idx,ny*(i-1)+j)
 90   continue

      spf2 = s

      return
      end

