c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: bemsrc.F,v 1.2 2001/04/06 11:21:10 nakano Exp $
c
c     routines to calculate the effect of internal sources on 
c     boundary nodes for BEM calculation
c
c     === common cbsrc ===
c     nn    ... number of nodes
c     ri(n) ... representation point of each node, r comp. 
c     zi(n) ...                                    z
c     bv(n) ... effect of sources on each node
c
#include "hisput.h"

      subroutine setbemsrc (n, r, z)
c
c     initializaton function for bemsrc
c
c     === input ===
c     n      ... number of boundary elements
c     r(n+1) ... R coordinate of the i-th boundary node
c     z(n+1) ... Z coordinate of the i-th boundary node
c
      integer n
      real*8 r(n+1), z(n+1)

      integer nodemax, nn
      parameter (nodemax = NODEMAX)
      double precision ri(nodemax), zi(nodemax), bv(nodemax)
      common /cbsrc/ ri, zi, bv, nn

      integer i

      if (n .gt. nodemax) then
         write(*,1000) nodemax
         stop
      end if
 1000 format(1x, '(pnsrc) -- node number should be less than ', I5)

      nn = n
      do 10 i = 1, n
         ri(i) = 0.5d0 * dble(r(i) + r(i+1))
         zi(i) = 0.5d0 * dble(z(i) + z(i+1))
         bv(i) = 0.0d0
 10   continue
      return
      end
c============================

      subroutine bemsrc (rs, zs)
c
c     calculate the contribution of delta-functional src at x
c     to each boundary nodes b
c
c     === input ===
c     rs, zs   ... src point (r,z)
c
      real*8 rs, zs

      integer nodemax, nn
      parameter (nodemax = NODEMAX)
      double precision ri(nodemax), zi(nodemax), bv(nodemax)
      common /cbsrc/ ri, zi, bv, nn

      real*8  ts, pi
      parameter (pi = PIDBL)
      integer i

      do 10 i = 1, nn
#ifdef DEBUG
         write(*,*) 'src:', rs, zs, ri(i), zi(i)
#endif
         bv(i) = bv(i) + ts(rs, zs, ri(i), zi(i)) / (2*pi)
 10   continue
      return
      end
c======

      subroutine readbemsrc(n, b)
c
c     read out the summary of bemsrc
c
c     === input ===
c     n      ... number of boundary elements
c
c     === output ===
c     b(n)   ... effect of sources at n-th boundary
c
      integer n
      real*8 b(n)

      integer nodemax, nn
      parameter (nodemax = NODEMAX)
      double precision ri(nodemax), zi(nodemax), bv(nodemax)
      common /cbsrc/ ri, zi, bv, nn

      integer i

      if (n .ne. nn) then
         write(*,1000) nn
         stop
      end if
 1000 format (1x, '(rnsrc) - node number is different from that ',
     $             'given in pnsrc() :', I5)

      do 10 i = 1, n
         b(i) = bv(i)
 10   continue
      return
      end
