c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: cdir.F,v 1.4 2001/01/13 02:41:21 nakano Exp $
c
c     routines for change the direction of sputtered particles
c     on collision in lab. system
c
c     common block cscatt
c       emr2p   .. 2 * unit charge / mass of sputt. particle (C/kg)
c       emr2mu  .. 2 * (unit charge) / (mmu)
c       rmp     .. mp / (mp+mg)
c       rmg     .. mg / (mp+mg)
c       bmax    .. maximum collision parameter (angstrom)
c
c       where mmu = mp * mg / (mp+mg) (kg) 
c
#include "hisput.h"

      subroutine cdir(ep, vdir, vg, vr)
c
c     determine new velocity of sputtered particles on collision event
c
c     input and output
c       ep     .. energy of sputtered particles (eV)
c       vdir   .. dirction of sputtered particles
c                 vdir(1) = n_vp(x) = sin(theta)cos(phi)
c                 vdir(2) = n_vp(y) = sin(theta)sin(phi)
c                 vdir(3) = n_vp(z) = cos(theta)
c       (theta [0..pi], phi [0..2pi])
c
c     input
c       vg     .. velocity of colliding gas (m/s)
c       vr     .. relative velocity of gas ang sputt. particle (m/s)
c
      real*8 ep, vdir(3), vg, vr
      real*8 genrand

      real*8 er, vp
      real*8 bc, sangle
      real*8 vvp(3), vvm(3), vvr(3)
      real*8 cts, cts2, sts, ps
      real*8 vp2, vg2, vr2
      real*8 ctr
      real*8 ctg, ctg2, stg
      real*8 pr, pp
      real*8 ctp, ctp2, stp, cpp, spp
      integer i

      real*8 emr2p, emr2mu, rmp, rmg, bmax
      common /ccdir/ emr2p, emr2mu, rmp, rmg, bmax

      real*8 pi
      parameter (pi = PIDBL)
#ifdef DEBUG
      real*8 vdbg, vdbg2
#endif

c
c     fisrt, det. vvr' in space III
c
      er = vr**2 / emr2mu
      
      bc = bmax * dsqrt(genrand())
      call scat(er, bc, sangle)
      cts = cos(sangle)

      cts2 = cts**2
      if (cts2 .ge. 1.0d0) then
         sts = 0.0d0
      else
         sts = sqrt(1.0d0 - cts2)
      end if
      ps = 2 * pi * genrand()
#ifdef DEBUG
      write(0,*) '==BEGIN cdir DEBUG MSG=='
      write(0,9000) cts, sts
 9000 format('scatt: ', 2e12.5)
#endif

      vvr(1) = vr * sts * cos(ps)
      vvr(2) = vr * sts * sin(ps)
      vvr(3) = vr * cts
c
c     determine cos_R and trans vvr' III -> II'
c
      vp = sqrt(ep * emr2p)
      vp2 = vp * vp
      vr2 = vr * vr
      vg2 = vg * vg
      if (vp .le. 0.0d0) then
         ctr = 1.0d0
      else
         ctr = (vp2 + vr2 - vg2) / (2 * vp * vr)
      end if
      call roty(vvr, ctr)

c
c     det. theta_g, then vvm, vvp' in II'
c
      if (vp * vg .le. 0.0d0) then
         ctg = 1.0d0
      else
         ctg = (vp2 + vg2 - vr2) / (2 * vp * vg)
      end if

      ctg2 = ctg**2
      if (ctg2 .ge. 1.0) then
         stg = 0.0d0
      else
C
C     since (str > 0) is implicitly assumed in roty...
C
         stg = - sqrt(1.0d0 - ctg2)
      end if

      vvm(1) = rmg * vg * stg
      vvm(2) = 0.0d0
      vvm(3) = rmp * vp + rmg * vg * ctg

      do 10 i = 1, 3
         vvp(i) = vvm(i) + rmg * vvr(i)
 10   continue

#ifdef DEBUG
 9010 format(1x, a4, e12.5, ':', 3e12.5)

      write(0,9010) 'vp0:', vp, 0, 0, vp
      write(0,9010) 'vg0:', vg,  vg*stg, 0, vg*ctg
      write(0,9010) 'vR0:', vr, -vg*stg, 0, vr*ctr

      vdbg2 = 0.0d0
      do 910 i = 1, 3
         vdbg2 = vdbg2 + vvm(i)**2
 910  continue
      vdbg = sqrt(vdbg2)
      write(0,9010) 'vG :', vdbg, (vvm(i),i=1,3)

      vdbg2 = 0.0d0
      do 920 i = 1, 3
         vdbg2 = vdbg2 + vvp(i)**2
 920  continue
      vdbg = sqrt(vdbg2)
      write(0,9010) 'vp1:', vdbg, (vvp(i),i=1,3)

      vdbg2 = 0.0d0
      do 930 i = 1, 3
         vdbg2 = vdbg2 + vvr(i)**2
 930  continue
      vdbg = sqrt(vdbg2)
      write(0,9010) 'vR1:', vdbg, (vvr(i),i=1,3)

      write(0,*) '==END cdir DEBUG MSG=='
#endif

c
c     rotate with phi_r ( II' -> II)
c
      pr = 2 * pi * genrand()
      call rotz(vvp, cos(pr), sin(pr))

c
c     finally, II -> I
c
      if (ep .le. 0.0d0) then
         ctp = 1.0d0 - 2 * genrand()
         pp = 2 * pi * genrand()
         cpp = cos(pp)
         spp = sin(pp)
      else
         ctp = vdir(3)
         
         ctp2 = ctp**2
         if (ctp2 .ge. 1.0d0) then
            cpp = 1.0d0
            spp = 0.0d0
         else
            stp = sqrt(1.0d0 - ctp2)
            cpp = vdir(1) / stp
            spp = vdir(2) / stp
         end if
      end if
      call roty(vvp, ctp)
      call rotz(vvp, cpp, spp)

c
c     det. ep and vdir after scatt.
c
      vp2 = 0.0d0
      do 20 i = 1, 3
         vp2 = vp2 + vvp(i)**2
 20   continue

      if (vp2 .eq. 0) then
         ep = 0
         return
      end if

      vp = sqrt(vp2)
      do 30 i = 1, 3
         vdir(i) = vvp(i) / vp
 30   continue

      ep = vp2 / emr2p

      return
      end


      subroutine setcdir(mp, mg, rmax)
c
c     initialization program for cdir
c
c     ==input==
c        mp     .. mass of particle (kg)
c        mg     .. mass of gas (kg)
c        rmax   .. maximum collision parameter(m)
c
      real*8 unitch
      parameter (unitch = UNITCH)

      real*8 mp, mg, rmax
      real*8 mmu
      real*8 emr2p, emr2mu, rmp, rmg, bmax
      common /ccdir/ emr2p, emr2mu, rmp, rmg, bmax

      rmp = mp / ( mp + mg )
      rmg = mg / ( mp + mg )

      mmu = rmp * mg
      emr2mu = 2 * unitch / mmu
      emr2p  = 2 * unitch / mp

      bmax = rmax * 1.0D10

      return
      end


      subroutine roty(v, ct)
c
c     reverse rotation of v with y axis
c
c     inputs:
c       ct .. cos(theta)>0, assumed to be theta < 0
c       v  .. 3D vector to be rotated and output
c
      real*8 v(3), ct
      real*8 ct2, st, v1, v3

      ct2 = ct**2
      if (ct2 .ge. 1.0d0) then
         st = 0.0d0
      else
         st = sqrt(1.0d0 - ct2)
      end if
      v1 = v(1)
      v3 = v(3)

      v(1) =   v1 * ct + v3 * st
      v(3) = - v1 * st + v3 * ct

      return
      end


C===
      subroutine rotz(v, cp, sp)
c
c     rotation of v with z axis
c
c     inputs:
c       cp .. cos(phi)
c       sp .. sin(phi)
c          where phi is polar angle
c       v is 3D vector to be rotated and output
c
      real*8 v(3), cp, sp
      real*8 v1, v2

      v1 = v(1)
      v2 = v(2)

      v(1) =   v1 * cp - v2 * sp
      v(2) =   v1 * sp + v2 * cp

      return
      end


C===
      subroutine relv(ep, vg, wr, vrel)
c
c     return Relative speed V(m/s) for gas & particle speed
c     with generated random number
c
c     ==Inputs==
c       ep ... k. energy of sputtered particle (eV)
c       vg ... speed of ambient gas        (m/s)
c       wr ... random nuber [0..1)
c
c     ==Output==
c       vrel .. relative speed between the particle and gas (m/s)
c
      real*8 ep, vg, wr, vrel
      real*8 vp, vu, vl, vrel3

      real*8 emr2p, emr2mu, rmp, rmg, bmax
      common /ccdir/ emr2p, emr2mu, rmp, rmg, bmax

      vp = sqrt(ep * emr2p)
      vu = vp + vg
      vl = abs(vp - vg)

      vrel3 = wr * vu**3 + (1.0d0 - wr) * vl**3
#ifdef DEBUG
      write(0,*) 'vrel', vu, vl, vrel3
#endif
      vrel = vrel3 ** (1.d0 / 3)

      return
      end

