c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 2000 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: ecross.f,v 1.6 2000/03/21 15:56:35 nakano Exp $
c
c     routines to obtain whether the particle pass crosses
c     with the line element in r-z plane of the axisymmetric system.
c
c     == common block ccross ==
c     all members are calculated from p1(3), p2(3), which are
c     the begin & end of the particle path, respectively.
c     

      subroutine scross(p1, p2)
c
c     setup routine for ecross.
c     
c     == input ==
c     p1(3) start point of particle path
c     p2(3) end point of particle path
c
      real*8 p1(3), p2(3)

      real*8 p1r, p2r, p1z, p2z, prip, prlen
      real*8 prmin, prmax, pzmin, pzmax
      common /ccross/ p1r, p1z, p2r, p2z, prip, prlen,
     $     prmin, prmax, pzmin, pzmax
      real*8 apex

      p1r = dsqrt(p1(1)**2 + p1(2)**2)
      p2r = dsqrt(p2(1)**2 + p2(2)**2)
      p1z = p1(3)
      p2z = p2(3)
      prip = p1(1)*p2(1) + p1(2)*p2(2)
      prlen = p1r**2 + p2r**2 - 2 * prip

      if (prlen .le. 0.0d0) then

c     case p1(i)==p2(i) (i=1,2)

         prmin = p1r
         prmax = p1r
      else
         apex = (p1r**2 - prip) / prlen

         if ((apex .ge. 0.0d0) .and. (apex .le. 1.0d0)) then
            prmin = dsqrt( (p1r**2 * p2r**2 - prip**2) / prlen)
         else
            prmin = dmin1(p1r, p2r)
         end if
         prmax = dmax1(p1r, p2r)
      end if
      pzmin = dmin1(p1z, p2z)
      pzmax = dmax1(p1z, p2z)

      return
      end


      subroutine ecross(e1r, e1z, e2r, e2z, ov, nr, t)
c
c     determine the crossing point of the line p1 -> p2
c     (in catesian coord., given via subroutine scross),
c     with the node element of (r1, z1)-(r2, z2)
c     (in axisymmetric coord.).
c     
c     == input ==
c     e1r, e1z:  r & z components of 1st node of the elem.
c     e2r, e2z:                      2nd
c     ov : whether the overwrap is counted (0) or not (1)
c
c     == output ==
c     nr : number of roots t.
c     t(2) : i-th root
c
c     where the crossing point is given by p1*t + p2*(1-t).
c     Therefore, t(i) should be [0..1] if they're valid.
c
c     BEFORE CALLING THIS ROUTINE, scross SHOULD BE CALLED
c
      real*8 e1r, e1z, e2r, e2z
      integer ov, nr
      real*8 t(2)

      real*8 p1r, p2r, p1z, p2z, prip, prlen
      real*8 prmin, prmax, pzmin, pzmax
      common /ccross/ p1r, p1z, p2r, p2z, prip, prlen,
     $     prmin, prmax, pzmin, pzmax

      real*8 ermin, ermax, ezmin, ezmax
      real*8 s, etr, etz, eramp

      real*8 disc, ah, bh, ch
      integer i

c
c     START
c
      ermin = dmin1(e1r, e2r)
      ermax = dmax1(e1r, e2r)
      ezmin = dmin1(e1z, e2z)
      ezmax = dmax1(e1z, e2z)

c     default value
      nr = 0
      t(1) = -1
      t(2) = -1

      if (prmax .lt. ermin .or.
     $     prmin .gt. ermax .or.
     $     pzmax .lt. ezmin .or.
     $     pzmin .gt. ezmax) then
         return
      end if

      if (prlen .le. 0.0d0) then
c     case p1(i)==p2(i) (i=1,2)

         if (p1z .eq. p2z) return

         if (e1r .eq. e2r) then
            if (ov .eq. 1) return

            if (p1z .lt. ezmin) then
               t(1) = (ezmin - p1z) / (p2z - p1z)
               nr = nr + 1
            else if (p1z .gt. ezmax) then
               t(1) = (ezmax - p1z) / (p2z - p1z)
               nr = nr + 1
            end if

            if (p2z .lt. ezmin) then
               t(2) = (ezmin - p1z) / (p2z - p1z)
               nr = nr + 1
            else if (p2z .gt. ezmax) then
               t(2) = (ezmax - p1z) / (p2z - p1z)
               nr = nr + 1
            end if
            call rcross(nr, t)
            return
         end if

         s = (p1r - e1r) / (e2r - e1r) 
         etz = (1 - s) * e1z + s * e2z
         if (pzmin .gt. etz .or. pzmax .lt. etz) return

         nr = 1
         t(1) = (etz - p1z) / (p2z - p1z)
         t(2) = -1
         return
      end if


      if (ermin .eq. ermax) then
         ah = prlen
         bh = - (p1r**2 - prip)
         ch = p1r**2 - ermin**2 
         disc = bh**2 - ah * ch

         if (disc .lt. 0.0d0) return

         t(1) = (-bh - dsqrt(disc)) / ah
         t(2) = (-bh + dsqrt(disc)) / ah
         
         do 10 i = 1, 2
            if (t(i) .lt. 0.0d0 .or. t(i) .gt. 1.0d0) then
               t(i) = -1
            else
               etz = (1-t(i)) * p1z + t(i) * p2z
               if (etz .lt. ezmin .or. etz .gt. ezmax) t(i) = -1
            end if
 10      continue

         call rcross(nr, t)
         return
      end if

                        
      if (ezmin .eq. ezmax) then
         if (p1z .eq. p2z) then
C
C     Overwrapping: there may be 4 roots at most,
C     but the case either for ermax or prmax is enough (prmax > ermin).
C     Other roots will be caught by neibours.
C
            if (ov .eq. 1) return
            etr = dmin1(ermax, prmax)

            ah = prlen
            bh = -(p1r**2-prip)
            ch = p1r**2 - etr**2 
            disc = bh**2 - ah * ch

            if (disc .lt. 0) return

            t(1) = (-bh - dsqrt(disc)) / ah
            t(2) = (-bh + dsqrt(disc)) / ah

            do 20 i = 1, 2
               if (t(i) .lt. 0.0d0 .or. t(i) .gt. 1.0d0) t(i) = -1
 20         continue

            call rcross(nr, t)
            return
         end if

         t(1) = (ezmin - p1z) / (p2z - p1z)
         etr = dsqrt((1-t(1))**2 * p1r**2 + t(1)**2 * p2r**2
     $        + 2 * t(1) * (1-t(1)) * prip)
         if (etr .lt. ermin .or. etr .gt. ermax) t(1) = -1
         t(2) = -1
         call rcross(nr, t)
         return
      end if


      eramp = (e2r - e1r) / (e2z - e1z)
      ah = eramp**2 * (p2z-p1z)**2 - p1r**2 - p2r**2 + 2*prip
      bh = eramp**2 * (p2z-p1z) * (p1z - e1z + e1r / eramp)
     $     + p1r**2 - prip
      ch = eramp**2 * (p1z - e1z + e1r / eramp)**2 - p1r**2

      disc = bh**2 - ah * ch

      if (disc .lt. 0) return
      
      t(1) = (-bh - sqrt(disc)) / ah
      t(2) = (-bh + sqrt(disc)) / ah

      do 30 i = 1, 2
         if (t(i) .lt. 0.0d0 .or. t(i) .gt. 1.0d0) then
            t(i) = -1
         else
            etz = (1-t(i)) * p1z + t(i) * p2z
            if (etz .lt. ezmin .or. etz .gt. ezmax) t(i) = -1
         end if
 30   continue

      call rcross(nr, t)
      return
      end


      subroutine rcross (nr, t)
c
c     sort t(i) and determin nr
c
      integer nr
      real*8 t(2)
      
      if (t(1) .eq. -1 .and. t(2) .eq. -1) then
         nr = 0
      else if (t(1) .eq. -1) then
         nr = 1
         t(1) = t(2)
         t(2) = -1
      else if (t(2) .eq. -1) then
         nr = 1
      else
         nr = 2
      end if
      return
      end

