c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: eeject.f,v 1.7 2001/02/23 05:38:35 nakano Exp $
c
c     common block ceej
c        femax: maximum of the Thompson's distribution function 
c        eemax: energy at which dist. func becomes zero.
c
c     !!!!!!!!!!! NOTE !!!!!!!!!!!
c     If eb <= 0.0 is given to seteej, it is assumed that
c     sputtered particles have constant energy e1.
c     This is represented by femax = 0, while eemax is set to e1.
c

      subroutine eeject(ee)
c
c     == output ==
c       ee ...ejection energy (eV)
c
      real*8 ee

      real*8 fe, genrand
      real*8 rfe

      real*8 femax, eemax
      common /ceej/ femax, eemax

      if (femax .le. 0.0) then
         ee = eemax
         return
      end if

 10   continue
        rfe = femax * genrand()
        ee = eemax * (1 - genrand())
      if (fe(ee) .lt. rfe) go to 10

      return
      end
      
c=====
      subroutine seteej(eb, e1, mt, mi)
c
c     initialization routine for function eeject
c
c     == inputs ==
c       eb    ... binding energy of the target material (eV)
c       e1    ... kinetic energy of injecting ion (eV)
c       mt    ... atomic mass of the target material
c       mi    ... atomic mass of injecting ion
c
c     NOTE: if eb <= 0.0, assume that sputtered particles have
c           constant energy e1
c
      real*8 eb, e1, mt, mi

      real*8 fe, dfe, efemax
      external dfe

      real*8 eps, eupper, elower
      parameter (eps = 1.0d-14)

      real*8 ceb, ce1gam, cfemax, ceemax
      common /ceej/ cfemax, ceemax
      common /cfej/ ceb, ce1gam

      if (eb .le. 0.0d0) then
         cfemax = 0.0d0
         ceemax = e1
         return
      end if

      ceb = eb
      ce1gam = e1 * ( 4*mi*mt / (mi+mt)**2 )
      ceemax = ce1gam - ceb

      elower = 0.1d0 * eb
      eupper = 2.0d0 * eb
      call bisctf(dfe, eupper, elower, eps, efemax)      

      cfemax = 1.1d0 * fe(efemax)

      return
      end

c=====
      double precision function fe(ee)
c
c     ejection energy distribution of sputtered particles,
c     based on the Thompson's formula.
c     M. W. Thompson, Philos. Mag. 18 (1968) 377.
c
      real*8 ee

      real*8 eb, e1gamm
      common /cfej/ eb, e1gamm

      if (ee .le. 0.0d0 .or. ee .gt. (e1gamm - eb)) then
          fe = 0.0d0
      else
         fe = ee * (1.d0-dsqrt( (eb+ee) / e1gamm ))
     $        / (ee + eb) ** 3
      end if

      return
      end

c=====
      double precision function dfe(ee)
c
c     derivative of fe
c
      real*8 ee

      real*8 ca, cb
      real*8 eb, e1gamm
      common /cfej/ eb, e1gamm

      ca = dsqrt( (ee + eb) / e1gamm )
      cb = 1.d0 + eb / ee
      dfe = 0.5d0 / (ca * e1gamm) * (ee**2 * cb**3)
     $     - (1.d0 - cb) * (2 * ee - eb) * cb**2

      return
      end

