c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 2000 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: inc.F,v 1.11 2001/01/05 20:09:29 nakano Exp $
c
c     routines to obtain the transit time of flying particle
c     in rectilinearly region for axisymmetric coordinates
c
c     == common block cincp ==
c     nfr            .. number of regions to calc the residual time.
c     rfr(4, ncb)    .. apexes of the region elements, r-coord.
c     zfr(4, ncb)    ..                                z
c     vfr(2, 4, nfr) .. 2D vector from i-th apex to
c                       counterclockwise neighbour
c     ifrbf(4, nfr)  .. flag if the boundary (i -> i+1) should be
c                       counted (0) or not (1)
c
c     == common block cince ==
c     tfr(nfr)       .. residual time. 
c     emr2p   .. 2 * unit charge / mass of sputt. particle (C/kg)
c
#include "hisput.h"

      subroutine setinc(mp)
c
c     initialization routine for trap
c
c     == input ==
c     mp      .. math of the particle (kg)
c
      real*8 mp

      real*8 unitch
      parameter (unitch = UNITCH)

      integer nfrmax, nfr
      parameter (nfrmax = REGMAX)
      real*8 rfr(4, nfrmax), zfr(4, nfrmax)
      real*8 vfr(2, 4, nfrmax)
      integer ifrbf(4, nfrmax)
      common /cincp/ nfr, rfr, zfr, vfr, ifrbf

      real*8 tfr(nfrmax)
      real*8 emr2p
      common /cince/ tfr, emr2p

      real*8 rotop

      integer ifr, ifrin
      integer j, k

 1910 format(1x, '(setinc) -- number of regions is too large')
 1920 format(1x, '(setinc) -- region numbering is corrupted')
 1930 format(1x, '(setinc) -- file open error, program terminated')

 2000  format(i10)
 2010  format(2f14.6, i2)

      emr2p = 2 * unitch / mp

      open(unit=IFD, status='old', file='region.dat', err=9000)

      read(IFD,2000) nfr
      if (nfr .gt. nfrmax) then
         write(0,1910)
         stop
      end if

      do 10 ifr = 1, nfr

         tfr(ifr) = 0.0d0

         read(IFD,2000) ifrin
         if (ifr .ne. ifrin) then
            write(0, 1920)
            stop
         end if

         do 11 j = 1, 4
            read(IFD,2010) rfr(j,ifr), zfr(j,ifr), ifrbf(j,ifr)
            rfr(j,ifr) = rfr(j,ifr) * 1.0d-2
            zfr(j,ifr) = zfr(j,ifr) * 1.0d-2
 11      continue

         rotop = (rfr(2,ifr)-rfr(1,ifr))*(zfr(3,ifr)-zfr(2,ifr))
     $         - (rfr(3,ifr)-rfr(2,ifr))*(zfr(2,ifr)-zfr(1,ifr))
         if(rotop .lt. 0) then
            call dswap(rfr(2,ifr), rfr(3,ifr))
            call dswap(zfr(2,ifr), zfr(3,ifr))
            call iswap(ifrbf(1,ifr), ifrbf(4,ifr))
            call iswap(ifrbf(2,ifr), ifrbf(3,ifr))
         end if

         do 12 j = 1, 4
            k = j+1
            if (k .eq. 5) k = 1
            vfr(1,j,ifr) = rfr(k,ifr) - rfr(j,ifr)
            vfr(2,j,ifr) = zfr(k,ifr) - zfr(j,ifr)
 12      continue

 10   continue
      close(IFD)
      return

 9000 write(0,1930)
      stop
      end

c------------------------------------------
      subroutine inc(p1, p2, ep)
c     
c     == input ==
c     
c     p1(3)   start point of particle path (m)
c     p2(3)   end point of particle path (m)
c     ep      kinetic energy of the particle during p1->p2 (eV)
c
c     == output ==
c     inside
c     time ... time the particle is present in the region
c
      real*8 p1(3), p2(3), ep

      integer nfrmax, nfr
      parameter (nfrmax = REGMAX)
      real*8 rfr(4, nfrmax), zfr(4, nfrmax)
      real*8 vfr(2, 4, nfrmax)
      integer ifrbf(4, nfrmax)
      common /cincp/ nfr, rfr, zfr, vfr, ifrbf

      real*8 tfr(nfrmax)
      real*8 emr2p
      common /cince/ tfr, emr2p

      real*8 ptrz(2)
      integer inner0

      integer it, nt, ntmax, kflag
      parameter (ntmax = 12)
      parameter (kflag = 1)
      real*8 t(ntmax), dummy(ntmax), tsum

      real*8 vp, rpath

      real*8 tc(2), tnn
      integer ntc

      integer ifr, iap, kap

      integer i

#ifdef DEBUG
      real*8 rt, zt
      character*4 seq
#endif

c
c     START
c

      call scross(p1, p2)

      vp = sqrt(ep * emr2p)

      rpath = 0.0d0
      do 10 i = 1, 3
         rpath = rpath + (p2(i)-p1(i))**2
 10   continue
      rpath = dsqrt(rpath)

      ptrz(1) = dsqrt(p1(1)**2 + p1(2)**2)
      ptrz(2) = p1(3)
      do 20 ifr = 1, nfr

         nt = 0
         do 21 iap = 1, 4
            kap = iap + 1
            if (kap .eq. 5) kap = 1

            call ecross(rfr(iap,ifr),zfr(iap,ifr),
     $           rfr(kap,ifr),zfr(kap,ifr),
     $           ifrbf(iap,ifr), ntc, tc)
#ifdef DEBUG
            write(6,3000), iap, rfr(iap,ifr), zfr(iap,ifr),
     $           rfr(kap,ifr), zfr(kap,ifr), (tc(i),i=1,2)
 3000       format(3x, 'at node', I2, ': (', 2f7.3, ')-(',
     $           2f7.3, ')  t:', 2f7.3)
#endif
            do 22 i = 1, ntc
               t(nt+i) = tc(i)
 22         continue
            nt = nt + ntc
 21      continue

         call pinner(ptrz, ifr, inner0)
         if (nt .gt. 0) call dsort(t, dummy, nt, kflag)

         if (t(1) .gt. 0.0d0 .and. inner0 .eq. 0) then
            do 23 it = 1, nt
               t(it+1) = t(it)
 23         continue
            t(1) = 0.0d0
            nt = nt + 1
         end if

C
C     -- sum cutting lengths --
C
         tsum = 0.0d0
         it = 1

 24      if (it .gt. nt) go to 25

         if (it+1 .gt. nt) then
            tsum = tsum + 1.0d0 - t(it)
            it = it + 1
         else if (t(it) .lt. t(it+1)) then
            tsum = tsum + t(it+1) - t(it)
            if (it+2 .le. nt .and. t(it+2) .eq. t(it+1)) then
               it = it + 1
            else
               it = it + 2
            end if
         else
            if (it+2 .le. nt) then
               tnn = 0.5d0 * (t(it+2) + t(it))
            else
               tnn = 0.5d0 * (1.0d0 + t(it))
            end if
            
            ptrz(1) = dsqrt(
     $           ( (1.0d0-tnn)*p1(1) + tnn*p2(1) )**2 +
     $           ( (1.0d0-tnn)*p1(2) + tnn*p2(2) )**2)
            ptrz(2) = (1.0d0-tnn)*p1(3) + tnn*p2(3)
            call pinner(ptrz, ifr, inner0)
            
            if (inner0 .eq. 0) tsum = tsum + 2 * (tnn - t(it))
            it = it + 3
         end if
         go to 24

 25      continue

#ifdef DEBUG
         do 510 it = 1, nt
            rt = dsqrt(
     $           ( (1-t(it))*p1(1) + t(it)*p2(1) )**2 +
     $           ( (1-t(it))*p1(2) + t(it)*p2(2) )**2)
            zt = (1-t(it))*p1(3) + t(it)*p2(3)
            if (t(it) .eq. t(it+1)) then
               seq = 'EQ'
            else
               seq = ''
            end if
            write(6,3010) it, t(it), rt, zt, seq
 510     continue
 3010    format(4x, I4, f10.7, ' (', 2f7.4, ')', A4)
         write(6,3020) tsum
 3020    format(4x, 'sum:', f10.7/)
#endif

         tfr(ifr) = tfr(ifr) + tsum * rpath / vp

 20   continue

      return
      end


c------------------------------------------
      subroutine repinc(ofd)
C
C     report the result:
C
C     ==input==
C     ofd : file descriptor for output.
C           it must be open before calling this routine
C
      integer ofd

      integer nfrmax, nfr
      parameter (nfrmax = REGMAX)
      real*8 rfr(4, nfrmax), zfr(4, nfrmax)
      real*8 vfr(2, 4, nfrmax)
      integer ifrbf(4, nfrmax)
      common /cincp/ nfr, rfr, zfr, vfr, ifrbf

      real*8 tfr(nfrmax)
      real*8 emr2p
      common /cince/ tfr, emr2p
      integer ifr, iap

 1000 format(1x, 'node:', I2, 'time:', E16.8)
 1010 format(1x, '  (', 2f10.6, ')')
 1020 format(/)
      do 10 ifr = 1, nfr
         write(ofd, 1000) ifr, tfr(ifr)
         do 20 iap = 1, 4
            write(ofd, 1010) rfr(iap, ifr), zfr(iap, ifr)
 20      continue
         write(ofd, 1020)
 10   continue

      return
      end


c------------------------------------------
      subroutine pinner(prz, ifr, inner)
c
c     subroutine to determine if the point is in the ifr-th region.
c     == INPUTS ==
c     prz    .. point (in r-z coordinate)
c     ifr    .. region index
c
c     == OUTPUT ==
c     inner  .. 0: p is in the ifr-th region
c               1: p is out of the region
c
      real*8 prz(2)
      integer ifr, inner

      integer nfrmax, nfr
      parameter (nfrmax = REGMAX)
      real*8 rfr(4, nfrmax), zfr(4, nfrmax)
      real*8 vfr(2, 4, nfrmax)
      integer ifrbf(4, nfrmax)
      common /cincp/ nfr, rfr, zfr, vfr, ifrbf

      real*8 op
      integer iapex

      inner = 1

      do 10 iapex =1, 4
         op = vfr(1,iapex,ifr) * (prz(2) - zfr(iapex,ifr))
     $        - vfr(2,iapex,ifr) * (prz(1) - rfr(iapex,ifr))
         if (ifrbf(iapex,ifr) .eq. 0) then
            if (op .lt. 0.0d0) return
         else
            if (op .le. 0.0d0) return
         end if
 10   continue

      inner = 0
      return
      end


c------------------------------------------
      subroutine iswap(ia, ib)
c
c     swap integers
c
      integer ia, ib, it

      it = ia
      ia = ib
      ib = it
      return
      end


c------------------------------------------
      subroutine dswap(da, db)
c
c     swap doubles
c
      real*8 da, db, dt

      dt = da
      da = db
      db = dt
      return
      end

