c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: itq.f,v 1.1.1.1 2000/01/06 05:47:17 nakano Exp $
c
c     integrant for axisymetric bem problems.
c
      subroutine preptq (ri, zi, rj0, zj0, rj1, zj1, lj)
c
c     set common block citq for ts and qs
c
      real*8 ri, zi, rj0, zj0, rj1, zj1, lj

      real*8 ric, zic, rj0c, rj1c, zj0c, zj1c, cnr, cnz
      common /citq/ ric, zic, rj0c, rj1c, zj0c, zj1c, cnr, cnz

      ric =  ri
      zic =  zi
      rj0c = rj0
      rj1c = rj1
      zj0c = zj0
      zj1c = zj1
      
      lj = dsqrt( (rj0 - rj1)**2 + (zj0 - zj1)**2 )
      cnr =   (zj1 - zj0) / lj
      cnz = - (rj1 - rj0) / lj

      return
      end
c-----------------------------------------------

      double precision function tsi(x)
c
c     integrant \theta^* rj
c
      real*8 x
      real*8 ri, zi, rj0, rj1, zj0, zj1, cnr, cnz
      common /citq/ ri, zi, rj0, rj1, zj0, zj1, cnr, cnz

      real*8 rj, zj
      real*8 ts

      rj = 0.5d0 * ( rj0 * (1.0d0-x) + rj1 * (1.0d0+x) )
      zj = 0.5d0 * ( zj0 * (1.0d0-x) + zj1 * (1.0d0+x) )

      if (rj .eq. 0.0d0) then
         tsi = 0.0d0
         return
      end if

      tsi = ts(ri, zi, rj, zj) * rj

      return
      end
c-----------------------------------------------

      double precision function ts(ri, zi, rj, zj)
c
c     function \theta^*
c
      real*8 ri, zi, rj, zj
      real*8 ca, cb, cc, cm, dellf

      ca = ri**2 + rj**2 + (zi - zj)**2
      cb = 2.0d0 * ri * rj

      if (ca .eq. cb) then
         write(6, 1000)
 1000    format(1x, '(ts)--zi=zj and ri=rj=0')
         stop
      end if

      cc = ca + cb
      cm = sqrt(2.0d0 * cb / cc)
      ts = 4.0d0 * dellf(cm) / dsqrt(cc)
c     write(*,*) '(ts): m & c', cm, cc

      return
      end
c-----------------------------------------------

      double precision function qsi(x)
c
c     integrant q^* rj
c
      real*8 x
      real*8 ri, zi, rj0, rj1, zj0, zj1, cnr, cnz
      common /citq/ ri, zi, rj0, rj1, zj0, zj1, cnr, cnz

      real*8 rj, zj, ca, cb, cc, cd, ce, cm, qs
      real*8 dellf, dells

      rj = 0.5d0 * ( rj0 * (1 - x) + rj1 * (1 + x) )
      zj = 0.5d0 * ( zj0 * (1 - x) + zj1 * (1 + x) )

      if (rj .eq. 0.0d0) then
         qsi = 0.0d0
         return
      end if
      
      if (ri .eq. 0.0d0) then
         write(6, 1000)
         stop
      end if
 1000 format(1x, '(qs)-- ri = 0')

      ca = ri**2 + rj**2 + (zi - zj)**2
      cb = 2 * ri * rj

      if (ca .eq. cb) then
         write(6, 1010)
         stop
      endif
 1010 format(1x, '(qsi)-- zi=zj and ri=rj')

      cc = ca + cb
      cm = sqrt(2 * cb / cc)
      cd = dells(cm)
      ce = ca - cb

      qs = ri**2 - rj**2 + (zi - zj)**2
      qs = qs / ce * cd - dellf(cm)
      qs = qs / (2 * rj) * cnr
      qs = qs + (zi - zj) / ce * cd * cnz
      qs = qs * 4 / dsqrt(cc)

      qsi = qs * rj
      
      return
      end
