c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998-2000 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: main.F,v 1.18 2002/04/09 17:25:42 nakano Exp $
c
#include "hisput.h"

      program hisput
c
c     main routine of HISPUT
c
c     == params ==
c     BASE
c       seed   ... seed for random generator sgenrand
c       ntrial ... trial number
c
c     PHYSICAL PARAMETERS
c       tgas   ... tempereture of the gas     (K)
c       ngas   ... gas density                (m^-3)
c       mgas   ... mass of gas atom           (kg)
c       mpar   ...         sputtered particle (kg)
c       rmax   ... maximum paremeter of the col-param (m)
c       ttherm ... thermalization threshold, in unit T of kT (K)
c
c     EJECTION
c       npej   ... nubmer of nodes to make the spline func. for
c                  errosion track
c       rpej   ... radial position of the erosion track node
c       dpej   ... depth of the erosion track node
c       zpej   ... z position of the erosio ntrack node.
c                  currently, it should be constant.
c
c       eb     ... binding energy of the target (eV)
c       e1     ... injection energy of gas ion to the target (eV)
c
c     SCATTERING
c       POTENTIAL PARAMETERS (see sp.f for details)
c         bmpca  ... Born-Mayer Potential parameter, A (eV)
c         bmpcb  ... Born-Mayer Potential parameter, B (angstrom^-1)
c
c       SPLINE FUNCTION PARAMETERS
c         nesca   ... number of nodes along energy axis
c         nbsca   ...                       col-param axis
c         emaxsca ... maximum value of energy axis (eV)
c         eminsca ... minimum
c         bmaxsca ... maximum value of col-param axis (angstrom)
c                     must be larger than rmax.
c
c     COLLISION
c       SPLINE FUNCTION PARAMETERS
c         necol   ... number of nodes along energy axis
c         nwcol   ...                       w(prob.)
c         emaxcol ... maximum of energy axis (eV)
c         emincol ... minimum
c         wmincol ... minimum of w axis, below which vg is prop. to w
c         wmaxcol ... maximun of w axis, above which vg is const.
c
c     BOUNDARY
c       nnode  ... number of boundary elements
c       rnode  ... radial positions of nodes of elements
c       znode  ... vertical positions
c       ibc    ... boundary conditions of elements (0:D, 1:N)
c

      integer seed
      integer*8 itrial, ntrial, neject

      logical cdensity

      real*8 tgas, ngas
      real*8 mgas, mpar, rmax
      real*8 ttherm

      real*8 eb, e1

      integer npejmax, npej
      parameter (npejmax = NMAXPEJ)
      real*8 rpej(npejmax), dpej(npejmax), zpej

      real*8 bmpca, bmpcb
      integer nesca, nbsca
      real*8 emaxsca, eminsca, bmaxsca

      integer necol, nwcol
      real*8 emaxcol, emincol, wmincol, wmaxcol

      integer nnodemax, nnode
      parameter(nnodemax = NODEMAX)
      real*8 rnode(nnodemax+1), znode(nnodemax+1)
      integer ibc(nnodemax)

      real*8 bv(nnodemax), bdv(nnodemax), b(nnodemax)
      real*8 bferr(nnodemax), bberr(nnodemax)
      real*8 g(nnodemax,nnodemax), h(nnodemax,nnodemax)
      integer ipd(nnodemax), iw(nnodemax)
      real*8 dw(3,nnodemax)
      real*8 ho(nnodemax,nnodemax), hp(nnodemax,nnodemax)

      real*8 genrand

      real*8 ep, vp(3), xp0(3), xp1(3), rp0, zp0
      real*8 vg, vr, fp, w

      integer nidx, ntrap(nnodemax)
      real*8 s, ftrap(nnodemax)
      real*8 etherm, unitbz, unitch
      parameter(unitbz = UNITBZ)
      parameter(unitch = UNITCH)

      integer i
c
c     start:
c
#ifdef FPE
      call setfpu
#endif

      call readin(seed, ntrial, cdensity,
     $     bmpca, bmpcb,
     $     nesca, nbsca, emaxsca, eminsca, bmaxsca,
     $     tgas, ngas, mgas, mpar, rmax, ttherm,
     $     necol, nwcol, emaxcol, emincol, wmaxcol, wmincol,
     $     nnodemax, nnode, rnode, znode, ibc,
     $     eb, e1,
     $     npejmax, npej, rpej, dpej, zpej)

      call sgenrand(seed)
      call inispf1
      call inispf2
      call hiab

      call seteej(eb, e1, mpar, mgas)
      call setpej(npej, rpej, dpej, zpej)

      if (cdensity .eqv. .true.) call setinc(mpar)

      call setsp(bmpca, bmpcb)
      call setsca(nesca, nbsca, emaxsca, eminsca, bmaxsca)
      call setcol(tgas, ngas, mgas, mpar, rmax,
     $     necol, nwcol, emaxcol, emincol, wmaxcol, wmincol)

      call setcdir(mpar, mgas, rmax)
      call settrap(nnode, rnode, znode)
      call setbemsrc(nnode, rnode, znode)

      do 5 i = 1, nnode
         ntrap(i) = 0
 5    continue

      neject = 0

      etherm = ttherm * unitbz / unitch

      do 10 itrial = 1, ntrial

         call peject(xp0)

 20      continue
         
#ifdef DEBUG
         write(0,9020)
 9020    format('new ejection')
#endif

         call eeject(ep)
         call aeject(vp)
         neject = neject + 1

 25      continue

c
c     -- particle path --
c
#ifdef DEBUG
         write(0,9000) ep
         write(0,9010) (vp(i),i=1,3)
 9000    format('ep:', g20.6)
 9010    format('vp:', 3g20.6)
#endif
         w = genrand()
         call frpath(ep, w, fp)
         do 30 i = 1, 3
            xp1(i) = xp0(i) + fp * vp(i)
 30      continue

         call trap(xp0, xp1, nidx)

         if (cdensity .eqv. .true.) call inc(xp0, xp1, ep)

         if (nidx .ne. 0 .and. ibc(nidx) .eq. 1) then
c
c     --  trapped on erosion track. re-eject --
c
            do 31 i = 1, 2
               xp0(i) = xp1(i)
 31         continue
            xp0(3) = 0.1d-9
            go to 20
         end if

         if (nidx .ne. 0 .and. ibc(nidx) .eq. 0) then
            write(6,1000) nidx, ep
 1000       format(1x, 'TRAPPED(IDX,E)', I4, f13.6)
            ntrap(nidx) = ntrap(nidx) + 1
            go to 10
         end if

         do 32 i = 1, 3
            xp0(i) = xp1(i)
 32      continue

c
c     -- collision --
c
         w = genrand()
         call colgas(ep, w, vg)
         w = genrand()
         call relv(ep, vg, w, vr)

#ifdef DEBUG
         write(0,9030) vg, vr
 9030    format('vg,vr:', 2g20.6)
#endif

         call cdir(ep, vp, vg, vr)

         if (ep .lt. etherm) then
            rp0 = dsqrt(xp0(1)**2 + xp0(2)**2)
            zp0 = xp0(3)
            call bemsrc(rp0, zp0)
            write(6,1010) rp0, zp0
 1010       format(1x, 'THERM(R,Z): ', 2f13.6)
            go to 10
         end if

         go to 25
 10   continue

c
c     -- TRAP summary --
c
      do 100 i = 1, nnode
         call axarea(rnode(i), znode(i), rnode(i+1), znode(i+1), s)
         ftrap(i) = ntrap(i) / s
 100  continue

c
c     -- BEM routine --
c
      do 200 i = 1, nnode
         bv(i) = 0.0d0
 200  continue
      call readbemsrc(nnode, b)
      
      call bem(nnodemax, nnode, rnode, znode, ibc, bv, bdv, b,
     $     g, h, ipd, bferr, bberr, dw, iw, ho, hp)

      write(OFD, 1020)
 1020 format(1x, '====RESULT (IDX, #trap, Fen, Fth===')

      do 300 i = 1, nnode
         write(OFD, 1030) i, ntrap(i), ftrap(i), bdv(i)
 300  continue  
 1030 format(1x, i4, i15, 2f20.6)

      if (cdensity .eqv. .true.) then
         call repinc(OFD)
      endif

      write (OFD, 1040) neject
 1040 format(1x, 'Total Ejection', i15)

      close(OFD)
      stop
      end
