c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: peject.F,v 1.4 2001/01/17 05:01:44 nakano Exp $
c
#include "hisput.h"

      subroutine peject(x)
c
c     routine to return the ejection point of a sputtered particle
c     in the axisymmetric coordination.
c
c     ==outputs==
c       x ... 3D position of the ejection.
c
      real*8 x(3)

      real*8 fpmax, pmin, pmax, z0
      integer idxpej
      common /cpej/fpmax, pmin, pmax, z0, idxpej

      real*8 genrand, spf1
      real*8 r0

      real*8 pi, phi
      parameter (pi = PIDBL)

 10   continue
         r0 = pmin + (pmax - pmin) * genrand()
      if ( fpmax * genrand() .ge. spf1(idxpej, r0) ) go to 10

      phi = 2 * pi * genrand()
      x(1) = r0 * cos(phi)
      x(2) = r0 * sin(phi)
      x(3) = z0

      return
      end

c-------------------
      subroutine setpej(npej, rpej, dpej, zpej)
c
c     initialization routine for peject
c     == inputs ==
c       npej       ... node number for depth prof. of the erosion track
c       rpej(npej) ... radial positions of erosion track node
c       dpej(npej) ... depth of erosion track at node npej
c       zpej       ... z position of the target surface
c
c     == params ==
c       kpej       ... order of spline func.
c       idpej      ... index for the 1d spline func.
c
      integer npej
      real*8 rpej(npej), dpej(npej), zpej

      real*8 fpmax, pmin, pmax, z0
      integer kpej, idpej, idxpej, idxdpej
      parameter (kpej = KSP1PEJ)
      parameter (idpej = ISP1PEJ)
      common /cpej/fpmax, pmin, pmax, z0, idxpej
      common /cdpej/ idxdpej

      integer i
      real*8 r0, r1, rm, d0, d1, dm, dd0, dd1
      real*8 spf1, dspf1, eps, ddpej
      external ddpej
      parameter (eps = 1.0d-5)

      pmin = rpej(1)
      pmax = rpej(npej)
      z0 = zpej
      idxpej = idpej
      idxdpej = idpej

      do 20 i = 1, npej
         dpej(i) = rpej(i) * dpej(i)
   20 continue

      call setspf1(idxpej, npej, kpej, rpej, dpej)

c     obtain maximum of spline func. for elimination method.
c     assumes spf1 has 1 inflection point at most (right?).
c
      fpmax = 0.0d0
      do 10 i = 1, npej-1
         r0 = rpej(i)
         r1 = rpej(i+1)
         d0 = dpej(i)
         d1 = dpej(i+1)
         dm = max(d0, d1)

         dd0 = dspf1(idxpej, r0)
         dd1 = dspf1(idxpej, r1)

         if (dd0 * dd1 .lt. 0.0d0) then
            call bisctf(ddpej, r0, r1, eps, rm)
            dm = max(dm, spf1(idxpej, rm))
         end if
         fpmax = max(fpmax, dm)
 10   continue

c     safety margin
c
      fpmax = 1.1d0 * fpmax
      return
      end
        
c------------
      double precision function ddpej(r)
c
c     derivative of depth profile func.
c
      real*8 r, dspf1
      integer idxpej
      common /cdpej/ idxpej

      ddpej = dspf1(idxpej, r)
      return
      end

