c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: trap.F,v 1.5 2001/01/17 07:58:30 nakano Exp $
c
c     routines to obtain the trapping boundary of the 
c     rectilinearly flying particle for axisymmetric coordinates
c
c     == common block ctrap ==
c     nb        .. number of boundary elemetns
c     rb(nb+1)  .. end point of boundary elements, r-coord.
c     zb(nb+1)  ..                                 z
c
#include "hisput.h"

      subroutine settrap(n, r, z)
c
c     initialization routine for trap
c
c     == input ==
c     n      .. number of boundary elements
c     r(n+1) .. r-coordinate of boundary node
c     z(n+1) .. z
c
      integer n
      real*8 r(n+1), z(n+1)

      integer nbmax, nb
      parameter (nbmax = NODEMAX)
      real*8 rb(nbmax+1), zb(nbmax+1)
      common /ctrap/nb, rb, zb

      integer i

      if (n .gt. nbmax) then
         write(*,1000) n, nbmax
         stop
      end if
 1000 format(1x, '(ptrap) - node nubmer ', I3,
     $      ' is greater than limit: ',I3)

      nb = n
      do 10 i = 1, n+1
         rb(i) = r(i)
         zb(i) = z(i)
 10   continue
      return
      end

c------------------------------------------
      subroutine trap(p1, p2, idx)
c
c     determine the boundary where the particle is trapped
c     
c     == input ==
c     p1(3) start point of particle path
c     p2(3) end point of particle path
c
c     == output ==
c     idx ... 0     = not trapped
c             other = index of trapped boundary element
c
c     p2  ... (if idx .ne. 0), trap point
c
      real*8 p1(3), p2(3)
      integer idx

      integer nbmax, nb
      parameter (nbmax = 100)
      real*8 rb(nbmax+1), zb(nbmax+1)
      common /ctrap/nb, rb, zb
      
      integer ov, nr
      real*8 tmin, t(2)
      integer i

      idx = 0
      ov = 0
      tmin = 2.0d0

      call scross(p1, p2)
      do 10 i = 1, nb
         call ecross(rb(i), zb(i), rb(i+1), zb(i+1), ov, nr, t)
         if (nr .eq. 0) go to 10

         if (nr .eq. 1) then
            if (t(1) .gt. 0.0d0 .and. t(1) .lt. tmin) then
               tmin = t(1)
               idx = i
            end if
            go to 10
         end if

         if (t(1) .gt. 0.0d0) then
            if (t(1) .lt. tmin) then
               tmin = t(1)
               idx = i
            end if
         else
            if (t(2) .lt. tmin) then
               tmin = t(2)
               idx = i
            end if
         end if
 10   continue

c
c     reset p2 if idx .ne. 0
c
      if (idx .ne. 0) then
         do 30 i = 1, 3
            p2(i) = (1-tmin) * p1(i) + tmin * p2(i)
 30      continue
      end if

      return
      end

