#!/usr/bin/perl
#
# $Id: summarize.pl,v 1.4 2001/04/06 11:50:57 nakano Exp $
#
use Statistics::Descriptive;

#
# defs
#
$nt = 2;
@tf = ("00K", "4hK");
@td = ("0.0", "400.0");

$np = 11;
@pf = (                                "0m50", "0m70",
       "1m10", "1m15", "1m20", "1m35", "1m50", "1m70",
       "2m10", "2m15", "2m20");
@pd = (                          0.5, 0.7,
         1.0,  1.5,  2.0,  3.5,  5.0, 7.0,
        10.0, 15.0, 20.0);

$ne = 3;
@ef = ("02", "05", "10");
@ed = (2.0, 5.0, 10.0);

$nk = 2;
@kf = ("B", "L");
@kd = ("B", "La");

$nrep = 10;

#
# read node positions
#
open IF, "$kf[0]$ef[0]-$tf[0]-$pf[0]/0.in";
while(<IF>){
    if (/# number of bundary element/){
	my @l = split;
	$nnode = $l[0];
 	last;
    }
}
while(<IF>){
    if (/# node class/){last;}
    chomp;
    my @l = split;
    push @rnode, $l[0];
    push @znode, $l[1];
}
close IF; 
for(my $inode=$nnode;$inode>0;$inode--){
    $rnode[$inode] = 0.5 * ($rnode[$inode] + $rnode[$inode-1]);
    $znode[$inode] = 0.5 * ($znode[$inode] + $znode[$inode-1]);
}
$rnode[0] = undef;
$znode[0] = undef;

#
# parse data
#
for(my $ie = 0; $ie < $ne; $ie++){
for(my $it = 0; $it < $nt; $it++){
for(my $ik = 0; $ik < $nk; $ik++){
for(my $ip = 0; $ip < $np; $ip++){

    my $dir = "$kf[$ik]$ef[$ie]-$tf[$it]-$pf[$ip]";

    print "$kd[$ik] $ed[$ie] eV $td[$it] K $pd[$ip] Pa\n";

    for($in = 1; $in <= $nnode; $in++){
	$flux[$in] = Statistics::Descriptive::Sparse->new();
    }

    for(my $irep = 0; $irep < $nrep; $irep++){

	open DF, "$dir/$irep.dat" or die "cannot open $dir/$irep.dat";

	while(<DF>){
	    if (/^ ====RESULT/){last;}
	}

	while(<DF>){
	    chomp;
	    my @line = split;
	    $in = $line[0];
	    $nodeflux = $line[2] + $line[3];
	    $flux[$in]->add_data($nodeflux);
	}
	close DF;
    }
   
    for($in = 1; $in <= $nnode; $in++){
	$flux_m[$ik][$ie][$it][$ip][$in] = $flux[$in]->mean();
	$flux_e[$ik][$ie][$it][$ip][$in] =
	    $flux[$in]->standard_deviation() / sqrt($flux[$in]->count());
#	print "$in: $flux_m[$ik][$ie][$it][$ip][$in] $flux_e[$ik][$ie][$it][$ip][$in]\n";
    }
}}}}

#
# output
#
system "mkdir -p posdep";

for($it=0;$it<$nt;$it++){
for($ie=0;$ie<$ne;$ie++){
    my $dfile = "$ef[$ie]-$tf[$it].dat";
    open DF, "> $dfile" or die "cannot open $dfile for writing\n";

    printf DF "#Pg(Pa)\t@%4.2fcm\t(pm)\t@%4.2fcm\t(pm)\t@%4.2fcm\t(pm)\n",
	$rnode[45], $rnode[43], $rnode[41];

    for($ip = 0; $ip < $np; $ip++){
#
# pressure dependence
#
	print DF "$pd[$ip]";
	for($in = 45; $in >= 41; $in-=2){
	    my $ratio = $flux_m[0][$ie][$it][$ip][$in]
		      / $flux_m[1][$ie][$it][$ip][$in] * 6;

	    my $error = $ratio * sqrt(
		($flux_e[0][$ie][$it][$ip][$in]
		/$flux_m[0][$ie][$it][$ip][$in])**2 +
		($flux_e[0][$ie][$it][$ip][$in]
		/$flux_m[0][$ie][$it][$ip][$in])**2);

	    printf DF "\t%7.5f\t%7.5f", $ratio, $error;
	}
	print DF "\n";
#
# position dependence
#
        my $pfile = "$ef[$ie]-$tf[$it]-$pf[$ip].dat";
        open PF, "> posdep/$pfile" or die "cannot open $pfile for writing";

#
#	comment out ratio graph...
#
#	print PF "#Pos(cm)\tRat\t(pm)\n";
#
#	for($in = 45; $in >= 38; $in--){
#	    my $ratio = $flux_m[0][$ie][$it][$ip][$in]
#		      / $flux_m[1][$ie][$it][$ip][$in] * 6;
#
#	    my $error = $ratio * sqrt(
#		($flux_e[0][$ie][$it][$ip][$in]
#		/$flux_m[0][$ie][$it][$ip][$in])**2 +
#		($flux_e[0][$ie][$it][$ip][$in]
#		/$flux_m[0][$ie][$it][$ip][$in])**2);
#
#	    printf PF "%7.2f\t%7.2f\t%7.2f\n", $rnode[$in], $ratio, $error;
#	}


	print PF "#node\tB\tdB\tLa\tdLa\n";
	for($in = 1; $in <= $nnode; $in++){
	    printf PF "%2d\t", $in;
	    printf PF "%10.3g\t", $flux_m[0][$ie][$it][$ip][$in];
	    printf PF "%10.3g\t", $flux_e[0][$ie][$it][$ip][$in];
	    printf PF "%10.3g\t", $flux_m[1][$ie][$it][$ip][$in];
	    printf PF "%10.3g\n", $flux_e[1][$ie][$it][$ip][$in];
	}
	close PF;
    }
    close DF;
}}

