#!/usr/bin/perl
#
# $Id: summarize.pl,v 1.8 2001/04/06 11:45:43 nakano Exp $
#
use Statistics::Descriptive;

#
# defs
#
$nt = 2;
@td = ("0.0", "400.0");
@tf = ("00K", "4hK");

$np = 15;
@pf = ("0m10", "0m15", "0m20", "0m35", "0m50", "0m70",
       "1m10", "1m15", "1m20", "1m35", "1m50", "1m70",
       "2m10", "2m15", "2m20");
@pd = (  0.1,  0.15, 0.2,  0.35, 0.5, 0.7,
         1.0,  1.5,  2.0,  3.5,  5.0, 7.0,
        10.0, 15.0, 20.0);

$nr = 2;
@rf = ("07cm", "13cm");
@rd = (7, 13);

$nrep = 10;

#
# read node positions
#
for(my $ir=0;$ir<$nr;$ir++){
    my @ri = ();
    my @zi = ();
    open IF, "$rf[$ir]-$tf[0]-$pf[0]/0.in";

    while(<IF>){
	if (/# number of bundary element/){
	    my @l = split;
	    $nnode[$ir] = $l[0];
	    last;
	}
    }

    while(<IF>){
	if (/# node class/){last;}
	chomp;
	my @l = split;
	push @ri, $l[0];
	push @zi, $l[1];
    } 
    close IF; 

    for(my $inode=$nnode[$ir];$inode>0;$inode--){
	$rnode[$ir][$inode] =
	    0.5 * ($ri[$inode] + $ri[$inode-1]);
	$znode[$ir][$inode] =
	    0.5 * ($zi[$inode] + $zi[$inode-1]);
    }
    $rnode[$ir][0] = undef;
    $znode[$ir][0] = undef;
}

#
# parse data
#
for(my $ir = 0; $ir < $nr; $ir++){
for(my $it = 0; $it < $nt; $it++){
for(my $ip = 0; $ip < $np; $ip++){

    print "$rd[$ir] cm $td[$it] K $pd[$ip] Pa\n";
    my $dir = "$rf[$ir]-$tf[$it]-$pf[$ip]";

    for($in = 1; $in <= $nnode[$ir]; $in++){
	$flux[$in] = Statistics::Descriptive::Sparse->new();
    }

    for(my $irep = 0; $irep < $nrep; $irep++){

	open DF, "$dir/$irep.dat" or die "cannot open $dir/$irep.dat";

	while(<DF>){
	    if (/^ ====RESULT/){last;}
	}

	while(<DF>){
	    chomp;
	    my @line = split;
	    $in = $line[0];
	    $nodeflux = $line[2] + $line[3];
	    $flux[$in]->add_data($nodeflux);
	}
	close DF;
    }
   
    for($in = 1; $in <= $nnode[$ir]; $in++){
	$flux_m[$ir][$it][$ip][$in] = $flux[$in]->mean();
	$flux_e[$ir][$it][$ip][$in] =
	    $flux[$in]->standard_deviation() / sqrt($flux[$in]->count());
#	print "$in: $flux_m[$ir][$it][$ip][$in] $flux_e[$ir][$it][$ip][$in]\n";
    }
}}}

#
# output
#
for(my $ir=0;$ir<$nr;$ir++){
for(my $it=0;$it<$nt;$it++){
    $dfile = "$rf[$ir]-$tf[$it].dat";
    open DF, "> $dfile" or die "cannot open $dfile for writing\n";

    print DF "#Pg(Pa)\t\@5mm\t(pm)\t\@10mm\t(pm)\n";

    for($ip = 0; $ip < $np; $ip++){
#
# pressure dependence
#
	print DF "$pd[$ip]";

        # 5mm
	#
        my $dom = $flux_m[$ir][$it][$ip][$nnode[$ir]-13]
                 +$flux_m[$ir][$it][$ip][$nnode[$ir]-14];

        my $num = $flux_m[$ir][$it][$ip][$nnode[$ir]-23]
	         +$flux_m[$ir][$it][$ip][$nnode[$ir]-24];

	my $ratio = $num / $dom;

        my $error = $ratio * sqrt(
		 ($flux_e[$ir][$it][$ip][$nnode[$ir]-13]/$dom)**2
		+($flux_e[$ir][$it][$ip][$nnode[$ir]-14]/$dom)**2
		+($flux_e[$ir][$it][$ip][$nnode[$ir]-23]/$num)**2
		+($flux_e[$ir][$it][$ip][$nnode[$ir]-24]/$num)**2);

	printf DF "\t%7.3e\t%7.3e", $ratio, $error;

        # 10mm
	#
        $dom = $flux_m[$ir][$it][$ip][$nnode[$ir]-11]
                 +$flux_m[$ir][$it][$ip][$nnode[$ir]-12];

        $num = $flux_m[$ir][$it][$ip][$nnode[$ir]-25]
	         +$flux_m[$ir][$it][$ip][$nnode[$ir]-26];

	$ratio = $num / $dom;

        $error = $ratio * sqrt(
		 ($flux_e[$ir][$it][$ip][$nnode[$ir]-11]/$dom)**2
		+($flux_e[$ir][$it][$ip][$nnode[$ir]-12]/$dom)**2
		+($flux_e[$ir][$it][$ip][$nnode[$ir]-25]/$num)**2
		+($flux_e[$ir][$it][$ip][$nnode[$ir]-26]/$num)**2);

	printf DF "\t%7.3e\t%7.3e\n", $ratio, $error;

#
# position dependence
#
        open PF, "> $rf[$ir]-$tf[$it]-$pf[$ip]/posdep.dat" or
		die "cannot open posdep.dat for writing\n";
	printf PF "#center flux: %f\n", $flux_m[$ir][$it][$ip][$nnode[$ir]];
	print PF "#R(cm)\tRat\t(pm)\n";

	for($in = $nnode[$ir]; $in > $nnode[$ir]-15 ; $in--){
	    my $ratio = $flux_m[$ir][$it][$ip][$in]
		      / $flux_m[$ir][$it][$ip][$nnode[$ir]];

	    my $error = $ratio * sqrt(
			 ($flux_e[$ir][$it][$ip][$in]
			 /$flux_m[$ir][$it][$ip][$in])**2
			+($flux_e[$ir][$it][$ip][$nnode[$ir]]
			 /$flux_m[$ir][$it][$ip][$nnode[$ir]])**2);
	    printf PF "%7.2f\t%7.3e\t%7.3e\n", $rnode[$ir][$in], $ratio, $error;
	}

	print PF "= = =\n";

	for($in = $nnode[$ir]-23; $in > $nnode[$ir]-27 ; $in--){
	    my $ratio = $flux_m[$ir][$it][$ip][$in]
		      / $flux_m[$ir][$it][$ip][$nnode[$ir]];

	    my $error = $ratio * sqrt(
			 ($flux_e[$ir][$it][$ip][$in]
			 /$flux_m[$ir][$it][$ip][$in])**2
			+($flux_e[$ir][$it][$ip][$nnode[$ir]]
			 /$flux_m[$ir][$it][$ip][$nnode[$ir]])**2);
	    printf PF "%7.2f\t%7.3e\t%7.3e\n", $rnode[$ir][$in], $ratio, $error;
	}

	close PF;
    }
    close DF;
}}

