#!/usr/bin/perl
#
# make graph to show the distributio nof thermalized points.
#
use PGPLOT;
# GLOBALS
$rawdisp = 0.15;
$coldisp = 0.22;

# mesh fineness (in m)
$mesh = 0.005;
$debug = "off";

$fmin = 0;
$fmax = 1000;

#
# draw graph using pgperl
#
pgopen("thermalize.ps/CPS");
pgwnad(0, 3 * $rawdisp, 0, 2.2*$coldisp);
pgsci(1);

#heat
@l=(0.0, 0.2, 0.4, 0.6, 1.0);
@r=(0.0, 0.5, 1.0, 1.0, 1.0);
@g=(0.0, 0.0, 0.5, 1.0, 1.0);
@b=(0.0, 0.0, 0.0, 0.3, 1.0);
pgctab(\@l, \@r, \@g, \@b, 5, 1, 0.5);

$wsize = 2;
$wdisp = -1;
$wwidth = 2;
pgsch($wsize);
pgwedg('TI', $wdisp, $wwidth, $fmin, $fmax, 'thermalized density (arb.)');

drawfile(0, 0, "13cm-4hK-1m15/0.out.bz2");
drawfile(0, 1, "13cm-4hK-1m20/0.out.bz2");
drawfile(0, 2, "13cm-4hK-1m35/0.out.bz2");
drawfile(1, 0, "13cm-4hK-1m50/0.out.bz2");
drawfile(1, 1, "13cm-4hK-1m70/0.out.bz2");
drawfile(1, 2, "13cm-4hK-2m10/0.out.bz2");

pgend;
exit;

#################################################################
# READ FILE FROM DATA AND DRAW IT
#################################################################
sub drawfile ($$$) {
    my ($ncol, $nraw, $his_out) = @_;
    my $xorg = $nraw * $rawdisp + 0.01;
    my $yorg = (1.4 - $ncol) * $coldisp;

    my $nnode, @rs, @zs;

    if ($his_out  =~ /bz2/){
	open OUTF, "bzip2 -dc $his_out |" || die "cannot open $his_out";
    } else {
	open OUTF, "his_out" || die "cannot open $his_out";
    }

    while(<OUTF>){
	if (/^ sputterd particle element/){
		chomp;
		my @line = split/:/;
		$elem = $line[1];
		$elem =~ s/ +$//;
	}

	if (/^ sputtering gas temperature/){
		chomp;
		my @line = split/:/;
		$tgas = $line[1] * 1;
	}

	if (/^ sputtering gas pressure \(gauge val\)/){
		chomp;
		my @line = split/:/;
		$pgas = $line[1] * 1;
	}

	if (/^ number of boundary elements/){
		chomp;
		my @line = split/ +: +/;
		$nnode = @line[1];
	}

	if (/^ number of erosion track elements/){last;}
	
	if(/^  node:/){
		chomp;
		my @line = split/:|\(|,|\)-\(|\)/;
		$inode = $line[1];
		if ($inode > $nnode){
			die "number of node exceed the declaration\n";
		}
		$rs[$inode] = $line[2];
		$zs[$inode] = $line[3];
		$re[$inode] = $line[4];
		$ze[$inode] = $line[5];
	}
    }

    $rmax = $rs[1]; if($rmax < $re[$nnode]){$rmax = $re[$nnode]};
    $zmax = $zs[1]; if($zmax < $ze[$nnode]){$zmax = $re[$nnode]};
    $zmin = $zs[1]; if($zmin > $ze[$nnode]){$zmin = $re[$nnode]};

    for($inode = 1; $inode < $nnode; $inode++){
	if($re[$inode] != $rs[$inode+1] || $ze[$inode] != $zs[$inode+1]){
		die "end of $inode th node differs from the start of next\n";
	}
	if($rmax < $re[$inode]){$rmax = $rs[$inode]};
	if($zmax < $ze[$inode]){$zmax = $ze[$inode]};
	if($zmin > $ze[$inode]){$zmin = $ze[$inode]};
    }

    if($debug eq "on"){
	print "rmax: $rmax\n";
	print "zmin: $zmin\n";
	print "zmax: $zmax\n";
    }

    $nr = int ( $rmax / $mesh + 0.5);
    $nz = int( ($zmax - $zmin) / $mesh + 0.5);

    if($debug eq "on"){
	print "nr: $nr\n";
	print "nz: $nz\n";
    }

    for(my $ir=0; $ir<$nr; $ir++){
	for(my $iz=0; $iz<$nz; $iz++){
		$count[$ir][$iz] = 0;
		$dens[$iz][$ir] = 0;
	}
    }

    while(<OUTF>){
	unless (/^ THERM\(R,Z\)/){next;}
	chomp;
	my @line = split;
	my $r = $line[1];
	my $z = $line[2];
	if ($r < 0 || $r > $rmax)    {print "$r exceeds boundary\n";}
	if ($z < $zmin || $z > $zmax){print "$z exceeds boundary\n";}
	my $ir = int($r / $mesh);
	my $iz = int( ($z - $zmin) / $mesh);
	$count[$ir][$iz]++;
    }

    my $dmax = 0;
    for(my $iz=0; $iz<$nz; $iz++){
	for(my $ir=0;$ir<$nr; $ir++){
	    if($debug eq "on"){
		printf "%2d %2d %6d\n", $ir, $iz, $count[$ir][$iz];
		printf "%2d %2d %20.6g\n", $ir, $iz, $dens[$ir][$iz];
	    }
	    my $volume = (2 * $ir + 1) * $mesh^2;
	    $dens[$iz][$ir] = $count[$ir][$iz] / $volume;
	    if ($dens[$iz][$ir] > $dmax) {$dmax = $dens[$iz][$ir]}
	}
	if($debug eq "on"){printf"\n";}
    }
    close OUTF;

    $b_bot   = $zmin - ($zmax - $zmin) * 0.10;
    $b_top   = $zmax + ($zmax - $zmin) * 0.00;
    $b_left  = 0;
    $b_right = $rmax;
#    pgwnad($b_left, $b_right, $b_bot, $b_top);

    @tr = ($xorg-$mesh/2, $mesh, 0, $yorg-$mesh/2+$zmin, 0, $mesh);

    pgimag(\@dens, $nr, $nz, 1, $nr, 1, $nz, $fmin, $fmax, \@tr);

    # draw frame
    #
    pgsci(0);
    pgslw(2); pgsls(1);
    pgmove($rs[1]+$xorg, $zs[1]+$yorg);
    for($inode=1;$inode<=$nnode;$inode++){
	pgdraw($re[$inode]+$xorg, $ze[$inode]+$yorg);
    }

    # draw hatch (HARDCORD!)
    #
    @area1r = (0, 0.045, 0.045, 0.06,  0.06,   0    );
    @area1z = (0, 0,    -0.02, -0.02, -0.045, -0.045);
    
    @area2r = (0,    0.04, 0.04,  0.01,  0.01, 0);
    @area2z = (0.05, 0.05, 0.065, 0.065, 0.12, 0.12);
    
    for(my $i = 0; $i < @area1r; $i++){
	$area1r[$i] = $xorg + $area1r[$i];
	$area1z[$i] = $yorg + $area1z[$i];
    }

    for(my $i = 0; $i < @area2r; $i++){
	$area2r[$i] = $xorg + $area2r[$i];
	$area2z[$i] = $yorg + $area2z[$i];
    }

    pgsfs(3); pgshs(60, 1, 0);
    pgsci(0); pgslw(1);
    pgpoly(6, \@area1r, \@area1z);
    pgpoly(6, \@area2r, \@area2z);

    # write text
    #
    my $textx = $xorg + 0.075;
    my $texty = $yorg - 0.08;
    my $tangle = 0;
    my $tjust = 0.5;
    $tsize = 2.0;
    $tfont = 1;

    $tstr = sprintf "%3.1fPa", $pgas;

    pgsci(1); pgsch($tsize); pgscf($tfont);
    pgptxt($textx, $texty, $tangle, $tjust, $tstr);
    pgsch(1);
}

