#!/usr/bin/perl
# $Id: summarize.pl,v 1.1 2001/01/26 14:06:20 nakano Exp $
# particle density summarizing script.
#
use PGPLOT;
use Statistics::Descriptive;

################### DEFINITIONS START#####################
#
# sputtering pressures
#
@pgas = (2, 5, 10, 20);
@prefix = ('020', '050', '100', '200');
$nrep = 5;

#
# consts for Cu
#
$roh   = 8.93;                # (g/cm^3)
$matom = 63.546 * 1.6605e-24; # (g/atom)

#
# experimentarl data (nm/min)
#
@depor = (10.46, 8.22, 9.92, 4.56);
@ddepo = (0.81, 0.76, 0.71, 0.48);

for($i=0; $i <= $#depor; $i++){
    $flux[$i] = $depor[$i] * 1e-7 / 60 * $roh / ($matom);
}
# flux is in unit (atom cm^-2 s^-1)

#
# regions
#
$kdots = 10;
$nr = $kdots; $nz = $kdots;
$hstep = 0.5e-2; # (m)
$rstep = 0.5e-2; # (m)
@tr = (-0.25, 0.5, 0, -0.25, 0, 0.5);

#
# density limits
#
$fmin = 16.5;
$fmax = 19;

#
# tag defs
#
$tdisp = 2;
$tpos = 0.5;
$tcord = 0.5;
$tsize = 7;
$tfont = 2;

#
# output filename
#
$figname = "density";
#################### DEFINITIONS END #####################

#
# assign region volumes
#
for(my $ir = 0; $ir < $nr; $ir++){
    my $pi = 4 * atan2(1.0, 1.0);
    $vol[$ir] = $pi * (2 * $ir + 1) * $rstep**2 * $hstep;
}

#
# parse calculation outputs
#
for (my $ip = 0; $ip<=$#prefix; $ip++){
    my $node = Statistics::Descriptive::Sparse->new();

    my @rtime;
    for(my $iz = 0; $iz < $nz; $iz++){
	for(my $ir = 0; $ir < $nr; $ir++){
	    my $i = $iz *10 + $ir;
	    $rtime[$i] = Statistics::Descriptive::Sparse->new();
	}
    }

    for ($irep = 1; $irep <= $nrep; $irep++){
	$fname = "$prefix[$ip]-$irep.dat";
	open DF, "$fname" or die "cannot open $fname\n";

        while(<DF>){if(/^ ====RESULT/){last;}}

        while(<DF>){
	    chomp;
	    @line = split;
	    if($line[0] == 44){
                $node->add_data($line[2]);
	    }
	    if($line[0] == 45){
                $node->add_data($line[2]);
		last;
	    }
	}
	while(<DF>){
	    chomp;
	    if(/^ node:/){
		@line = split / node: *|time: */;
		my $inode;
		if ($line[1] eq "**"){
		    $inode = 99;
		} else{
		    $inode = $line[1] - 1;
		}
#		print "$inode: $line[2]\n";
		$rtime[$inode]->add_data($line[2]);
	    }
	}
        close DF;
    }
    my $node_m  = $node->mean();
    my $node_e = $node->standard_deviation() / sqrt($node->count());

    my $stime_m = $node_m * 1e-4 / $flux[$ip];
    my $stime_e = $stime_m * sqrt( ($ddepo[$ip]/$depor[$ip])**2
			+ ($node_e/$node_m) **2);

    printf "CASE %2d Pa:\n", $pgas[$ip];
    printf "Sim Time (s): %14e (pm) %14e\n", $stime_m, $stime_e;
    printf " R Z density(m^-3)\n";

    for(my $iz=0; $iz < $nz; $iz++){
	for(my $ir=0; $ir < $nr; $ir++){
	    my $in = $iz*10+$ir;

	    my $rtime_m = $rtime[$in]->mean();
	    my $rtime_e = $rtime[$in]->standard_deviation()
			 / sqrt($rtime[$in]->count());

	    my $dens_m = $rtime_m / $stime_m / $vol[$ir];
	    my $dens_e = $dens_m * sqrt( ($stime_e/$stime_m)**2
				+ ($rtime_e/$rtime_m)**2);
	    printf "%2d %2d %14e (pm) %14e\n", $ir+1, $iz+1, $dens_m, $dens_e;
	    ${$d[$ip]}[$iz][$ir] = log($dens_m) / log(10);
        }
    }
}

pgopen("$figname.ps/CPS");
pgsci(1);

# examples from pgdemo4.f in /usr/doc/pgplot5/examples
#
#        -- gray scale
#      DATA GL /0.0, 1.0/
#      DATA GR /0.0, 1.0/
#      DATA GG /0.0, 1.0/
#      DATA GB /0.0, 1.0/
#pgctab(\@l, \@r, \@g, \@b, 2, 1, 0.5);

#        -- rainbow
#      DATA RL /-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
#      DATA RR / 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
#      DATA RG / 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
#      DATA RB / 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/
#
@l= (-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7);
@r= ( 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0);
@g= ( 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0);
@b= ( 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0);
pgctab(\@l, \@r, \@g, \@b, 9, 1, 0.5);

#        -- heat
#      DATA HL /0.0, 0.2, 0.4, 0.6, 1.0/
#      DATA HR /0.0, 0.5, 1.0, 1.0, 1.0/
#      DATA HG /0.0, 0.0, 0.5, 1.0, 1.0/
#      DATA HB /0.0, 0.0, 0.0, 0.3, 1.0/
#
#@l=(0.0, 0.2, 0.4, 0.6, 1.0);
#@r=(0.0, 0.5, 1.0, 1.0, 1.0);
#@g=(0.0, 0.0, 0.5, 1.0, 1.0);
#@b=(0.0, 0.0, 0.0, 0.3, 1.0);
#pgctab(\@l, \@r, \@g, \@b, 5, 1, 0.5);

#
# draw a wedge.
#
$wsize = 3;
$wdisp = -4.0 / $wsize;
$wwidth = 1.5;
pgsch($wsize);
pgwedg('TI', $wdisp, $wwidth, $fmin, $fmax, 'log10 density / m\u-3\d');

pgsubp(4,1);

#
# 1st page
#
pgpanl(1,1);
pgwnad(0, 5, -1, 7);
pgimag(\@{$d[0]}, $kdots, $kdots, 1, $kdots, 1, $kdots, $fmin, $fmax, \@tr);

&drawframe();

pgsch($tsize); pgscf($tfont);
pgmtxt('B', $tdisp, $tpos, $tcord, '2 Pa');
pgsch(1);

#
# 2nd page
#
pgpage;
pgwnad(0, 5, -1, 7);

pgimag(\@{$d[1]}, $kdots, $kdots, 1, $kdots, 1, $kdots, $fmin, $fmax, \@tr);

&drawframe();

pgsch($tsize); pgscf($tfont);
pgmtxt('B', $tdisp, $tpos, $tcord, '5 Pa');
pgsch(1);

#
# 3rd page
#
pgpage;
pgwnad(0, 5, -1, 7);

pgimag(\@{$d[2]}, $kdots, $kdots, 1, $kdots, 1, $kdots, $fmin, $fmax, \@tr);

&drawframe();

pgsch($tsize); pgscf($tfont);
pgmtxt('B', $tdisp, $tpos, $tcord, '10 Pa');
pgsch(1);

#
# 4th page
#
pgpage;
pgwnad(0, 5, -1, 7);

pgimag(\@{$d[3]}, $kdots, $kdots, 1, $kdots, 1, $kdots, $fmin, $fmax, \@tr);

&drawframe();

pgsch($tsize); pgscf($tfont);
pgmtxt('B', $tdisp, $tpos, $tcord, '20 Pa');
pgsch(1);

pgend;

#system("convert $figname.ps $figname.jpg");
exit;

sub drawframe{
    pgslw(2); pgsls(3);
    pgmove(0, -1);
    pgdraw(0,7);

    pgslw(3); pgsls(1);
    pgmove(0,0);
    pgdraw(4.5,0);
    pgdraw(4.5,-1);

    pgmove(0,5);
    pgdraw(4,5);
    pgdraw(4,6.5);
    pgdraw(1,6.5);
    pgdraw(1,7);
}

